#ifndef INC_TCPCLIENT_H
#define INC_TCPCLIENT_H
//-----------------------------------------------------------------------------
/*!
\file TcpClient.h
\brief
\if english
  Header for Client
\endif

\if german
  Header für Client
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpMain.h"
#include "TcpReceiverTransmitter.h"
#include <string>
#include <queue>
#include "TcpMessage.h"
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( disable : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( disable : 4275 )
  #endif
  //---------------------------------------------------------------------------
  //Forwards
  class ClientReceiver;
  class Transmitter;
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    A simple TCP client class, which should learn its know-how in the derived classes.

    Client runs as a thread that can connect to a socket server.

    Client has as superclass attributes the port, the data size which is sent,
    the address family, the socket type, the protocol type and a MainSocket (here
    the connection socket to the server).

    Additional attributes are a ClientReceiver, which processes the data from the
    server as a thread, a Transmitter, which sends data to the server, the socket
    address, the host name and a MsgQueue, which receives the individual messages
    of the ClientReceiver.

    Starting the client:
    ------------------
    During instantiation with transfer of host name, port, address family, socket
    type, protocol and optionally the data size, the socket address is first
    determined using the transferred host name and the own thread is started.

    Connection to the server:
    -------------------------
    An attempt is made to contact the server using the client method
    Client::connectServer() If this succeeds, the ClientReceiver and Transmitter
    are instantiated and started (these are also threads).

    The Client waits in Client::work() for data from the server (listens
    blockingly).

    Termination of the connection by the server:
    ----------------------------------------
    The socket is released in the ClientReceiver in ClientReceiver::work().
    If error < 0, then the connection is broken.

    Processing data from the server:
    -----------------------------
    A tcp::Message or derived message is created via
    ClientReceiver::generateNewTcpMessage (...)
    and forwarded to ClientReceiver::acceptMessage(AbstractMessage*).
    <br>
    This method can be overwritten; the ClientReceiver simply forwards the message
    to its parent (Client or subclass) Client::acceptServerMessage(). There,
    the message is inserted into the MsgQueue of the client and processed further
    in Client::work() and forwarded to the client method 
    Client::processServerMessage(Message *), which then deletes the message here;
    this method can also be overwritten.

   Sending to the server:
    ---------------------
    Sending is done via Client::send (...) (can be overwritten in subclasses)
    which packs the data here into a tcp::Message and passes it to
    Transmitter::acceptOrder (...) for further processing, which then sends this data
    to the server.

    Terminating the connection to the server:
    ---------------------------------
    Via Client::disconnectServer() the connection to the server is terminated.
    The ClientReceiver and Transmitter are removed. A new connect is possible.
  \endif

  \if german
    Eine einfache TCP-Clientklasse, welche ihr KnowHow in den abgeleiteten
    Klassen lernen soll.

    Client läuft als Thread, welcher Verbindung zu einem Socket-Server aufnehmen
    kann.

    Client hat als Oberklassen-Attribute den Port, die Datengröße welche versendet
    wird, die Adressfamilie, den Sockettyp, den Protokoltyp und einen MainSocket
    (hier den Verbindungssocket zum Server).

    Zusätzlich sind hier als Attribute ein ClientReceiver, welcher die Daten vom
    Server als Thread bearbeitet, ein Transmitter, welcher Daten an den Server sendet,
    die Socketadresse, der Hostname und eine MsgQueue, welche die einzelnen Messages
    des ClientReceiver aufnimmt.

    Start des Clients:
    ------------------
    Bei der instanziierung mit Übergabe von Hostname, Port, Adressfamilie, Sockettyp,
    Protokol und optional der Datengröße, wird zuerst einmal die Socketadresse anhand
    des übergebenen Hostnamen ermittelt und der eigene Thread gestartet.

    Mit dem Server verbinden:
    -------------------------
    Es wird versucht, mit dem Server über die Client-Methode Client::connectServer()
    Kontakt aufzunehem. Wenn dieses gelingt, werden der ClientReceiver und Transmitter
    instanziiert und gestartet (auch diese sind Threads).

    Der Client wartet in Client::work() auf Daten vom Server (lauscht
    blockierent).

    Abbruch der Verbindung durch den Server:
    ----------------------------------------
    Der Socket wird im ClientReceiver in ClientReceiver::work() freigegeben. Wenn
    Fehler < 0, dann ist die Verbindung abgebrochen.

    Daten vom Server verarbeiten:
    -----------------------------
    Es wird über ClientReceiver::generateNewTcpMessage (...)
    eine tcp::Message bzw. abgeleitete Message erstellt und an
    ClientReceiver::acceptMessage(AbstractMessage*) weitergegeben.
    <br>
    Diese Methode kann überschrieben werden; der ClientReceiver reicht die Message
    lediglich an seinen Parent (Client oder Unterklasse) Client::acceptServerMessage()
    weiter. Dort wird die Message in die MsgQueue des Clients eingefügt und in Client::work()
    weiterverarbeitet und an die Clientmethode 
    Client::processServerMessage(Message *) weitergeleitet, welche hier anschließend
    die Message löscht; auch diese Methode kann überschrieben
    werden.

    Senden an den Server:
    ---------------------
    Gesendet wird über Client::send (...) (kann in Unterklassen überschrieben werden)
    welche die Daten hier in eine tcp::Message packt und an Transmitter::acceptOrder (...)
    zum weiterverarbeiten übergibt, der diese Daten dann an den Server sendet.

    Beenden der Verbindung zu Server:
    ---------------------------------
    Über Client::disconnectServer() wird die Verbindung zum Server abgebaut. Es
    werden der ClientReceiver und Transmitter entfernt. Ein erneutes connect ist
    möglich.
  \endif

  \sa MyTestClient

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll Client : public TcpMain
  {
    protected:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The Receiver receives data from a Server for this client.
      \endif

      \if german
        Der Leser empfängt Daten von einem Server für diesen Client.
      \endif
      \sa ClientReceiver
      */
      ClientReceiver * m_pReceiver;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The Transmitter sends data to a Server for this client.
      \endif

      \if german
        Der Schreiber sendet Daten an einen Server für diesen Client.
      \endif
      \sa Transmitter
      */
      Transmitter * m_pTransmitter;
      //-----------------------------------------------------------------------

    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The address structure.
      \endif

      \if german
        Die Adressstruktur.
      \endif
      */
      SOCKADDR_IN m_Socketadress;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The host name or IP address (for example, "LOCALHOST" or "192.168.2.102").
      \endif

      \if german
        Der Hostname oder IP-Adresse (z.B. "LOCALHOST" oder "192.168.2.102").
      \endif
      */
      std::string m_sHostname;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The queue with the data received from the server.
      \endif

      \if german
        Die Queue mit den empfangenen Daten vom Server.
      \endif
      */
      std::queue<Message *> m_MsgQueue;
      //-----------------------------------------------------------------------
      /*!
      \if english
        If this value is not equal to ::ACTION_UNDEF, then in Client::work()
        any Message that does not have this action code will be appended back
        to the MsgQueue. Thus, Client::work() waits for the Message with this
        ActionCode.

        When the message with this action code arrives in Client::work(),
        #m_eToWaitingForServer is set to ::ACTION_UNDEF again and the message is
        forwarded to Client::processServerMessage (...) for further processing.
      \endif

      \if german
        Wenn dieser Wert ungleich ::ACTION_UNDEF, wird in Client::work() jede
        Message, welche nicht diesen ActionCode hat, wieder in die MsgQueue
        angehängt. Somit wird in Client::work() auf die Message mit diesem
        ActionCode gewartet.

        Wenn dann, in Client::work(), die Message mit dem diesem ActionCode
        eintrifft, wird #m_eToWaitingForServer wieder auf ::ACTION_UNDEF gesetzt
        und die Message zur Weiterverarbeitung an
        Client::processServerMessage (...) weiter gereicht.
      \endif
      */
      ActionCode m_eToWaitingForServer;
      //-----------------------------------------------------------------------

    protected:
      //============== Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Client();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Client( unsigned int,
              int,
              int,
              int,
              int );
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The constructor starts the client, but is not yet connected to the server
        from rsHostname.
        \param[in] rsHostname The host name or IP address (e.g. "LOCALHOST" or
          "192.168.2.102").
        \param[in] uiPort The port of the server.
        \param[in] iAdressFamily The socket address family to instantiate a socket
          (usually AF_INET).
        \param[in] iSocketType The socket type (usually SOCK_STREAM).
        \param[in] iProtocol The socket protocol (usually 0).
        \param[in] iSecTimeOut The time until waiting for a connection or waiting
          for data is aborted.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Der Konstruktor startet den Client, ist aber noch nicht mit dem Server aus
        rsHostname verbunden.
        \param[in] rsHostname Der Hostname oder IP-Adresse (z.B. "LOCALHOST" oder
          "192.168.2.102").
        \param[in] uiPort Der Port des Servers.
        \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
          instanziieren (i.d.R. AF_INET).
        \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM).
        \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
        \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
           warten auf Daten abgebrochen wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      Client( const std::string & rsHostname,
              unsigned int uiPort,
              int iAdressFamily,
              int iSocketType,
              int iProtocol,
              int iSecTimeOut );
      //-----------------------------------------------------------------------
      /*!
      \if english
        The constructor starts the client, but is not yet connected to the server
        from rsHostname.
        \param[in] rsHostname The host name or IP address (e.g. "LOCALHOST" or
          "192.168.2.102").
        \param[in] uiPort The port of the server.
        \param[in] iAdressFamily The socket address family to instantiate a socket
          (usually AF_INET).
        \param[in] iSocketType The socket type (usually SOCK_STREAM).
        \param[in] iProtocol The socket protocol (usually 0).
        \param[in] uiDataSize The packet size.
        \param[in] iSecTimeOut The time until waiting for a connection or waiting
          for data is aborted.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Der Konstruktor startet den Client, ist aber noch nicht mit dem Server aus
        rsHostname verbunden.
        \param[in] rsHostname Der Hostname oder IP-Adresse (z.B. "LOCALHOST" oder
          "192.168.2.102").
        \param[in] uiPort Der Port des Servers.
        \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
          instanziieren (i.d.R. AF_INET).
        \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM).
        \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
        \param[in] uiDataSize Die Paketgröße.
        \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
           warten auf Daten abgebrochen wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      Client( const std::string & rsHostname,
              unsigned int uiPort,
              int iAdressFamily,
              int iSocketType,
              int iProtocol,
              unsigned int uiDataSize,
              int iSecTimeOut );
      //-----------------------------------------------------------------------

       //============== Destruktore
      //-----------------------------------------------------------------------
      /*!
      \if english
        In the destructor the connection to the server is terminated if not already
        done.
      \endif

      \if german
        Im Destruktor wird die Verbindung zum Server abgebrochen wenn noch nicht
        geschehen.
      \endif
      */
      virtual ~Client();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Client( const Client & );
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Work of the thread. A received Message is forwarded to method
        Client::processServerMessage(Message * pTcpMessage).
      \endif

      \if german
        Arbeit des Threads. Es wird eine erhaltene Message an Methode
        Client::processServerMessage(Message * pTcpMessage) weitergereicht.
      \endif
      */
      virtual void work() override;
      //-----------------------------------------------------------------------
      #if defined (__PTHREADS)
      /*!
      \if english
        When the thread ends, this virtual method is called. The MsgQueue is
        cleaned up here.
        \note This method only exists when compiled with POSIX threads (defined
          (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        Hier wird die MsgQueue aufgeräumt.
        \note Diese Methode gibt es nur, wenn mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
      virtual void cleanUp();
      //-----------------------------------------------------------------------
      #else
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      /*!
      \if english
        When the thread ends, this virtual method is called. The MsgQueue is
        cleaned up here.
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        Hier wird die MsgQueue aufgeräumt.
      \endif
      */
      virtual void cancel() override;
      #endif
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a Transmitter here; can be overwritten in subclasses to provide a
        derived type of Transmitter if necessary.
        \return Pointer to the Transmitter to use (which sends data to a Server) for
          this client.
        \attention This method is used internally and the object instantiated on
          the heap is also internally removed from memory.
      \endif

      \if german
        Liefert hier einen Transmitter; kann in Unterklassen überschrieben werden
        um ggf. einen abgeleiteten Typ von Transmitter zu liefern.
        \return Zeiger auf den zu nutzenden Schreiber (dieser sendet Daten an einen
          Server) für diesen Client.
        \attention Diese Methode wird intern genutzt und das auf dem Heap instanziierte
          Objekt auch intern wieder aus dem Speicher entfernt.
      \endif
      */
      virtual Transmitter * generateNewTransmitter();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a ClientReceiver here; can be overwritten in subclasses to provide
        a derived type of ClientReceiver if necessary.
        \return Pointer to the Receiver to be used (which receives data from a
          Server) for this client
        \attention This method is used internally and the object instantiated on
          the heap is also internally removed from memory.
      \endif

      \if german
        Liefert hier einen ClientReceiver; kann in Unterklassen überschrieben
        werden um ggf. einen abgeleiteten Typ von ClientReceiver zu liefern.
        \return Zeiger auf den zu nutzenden Leser (dieser empfängt Daten von einem
          Server) für diesen Client
        \attention Diese Methode wird intern genutzt und das auf dem Heap instanziierte
          Objekt auch intern wieder aus dem Speicher entfernt.
      \endif
      */
      virtual ClientReceiver * generateNewReceiver();
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the packet size.
        \param[in] uiDataSize The package size.
      \endif

      \if german
        Setzt die Paketgröße.
        \param[in] uiDataSize Die Paketgröße.
      \endif
      */
      virtual void setDataSize( unsigned int uiDataSize ) override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Connects to the server.
        \return A return < 0 indicates an error.
      \endif

      \if german
        Verbindet sich mit dem Server.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int connectServer();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Terminates the connection with the server.
        \return A return < 0 indicates an error.
      \endif

      \if german
        Beendet die Verbindung mit dem Server.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      virtual int disconnectServer();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Indicates whether the server is connected.
        \return If true, there is a connection.
      \endif

      \if german
        Gibt Auskunft, ob mit dem Server verbunden.
        \return Wenn true, dann besteht eine Verbindung.
      \endif
      */
      bool isServerConnected() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sends the transferred data in a Message via its Transmitter to the server.
        \param[in] pszRawMsg Pointer to the data to be sent.
        \param[in] uiRawMsgSize The size of the data to be sent.
        \param[in] uiMessageID A MessageID can be passed.
        \param[in] iActionCode A ActionCode can be passed.
        \param[in] rsInfo A free info can be passed.
        \return A return value < 0 indicates an error.
        \attention The data in pszRawMsg is internally removed from memory; the
          calling instance may no longer use the data to be sent or remove it from
          memory.
      \endif

      \if german
        Sendet die übergebene Daten in einer Message über seinen Transmitter an
        den Server.
        \param[in] pszRawMsg Zeiger auf die zu sendenden Daten.
        \param[in] uiRawMsgSize Die Größe der zu sendenden Daten.
        \param[in] uiMessageID Es kann eine MessageID übergeben werden.
        \param[in] iActionCode Es kann ein ActionCode übergeben werden.
        \param[in] rsInfo Es kann eine freie Info übergeben werden.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Es werden die Daten in pszRawMsg intern aus dem Speicher entfernt;
          die aufrufende Instanz darf die zu sendenden Daten nicht mehr verwenden oder
          aus dem Speicher entfernen.
      \endif
      */
      virtual int send( char * pszRawMsg,
                        unsigned int uiRawMsgSize,
                        unsigned int uiMessageID = 0,
                        int iActionCode = 0,
                        const std::string & rsInfo = "" );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sends passed Message to the server via its Transmitter.
        \param[in] pTcpMessage The message packet to be sent.
        \return A return < 0 indicates an error.
        \attention The passed Message must not be removed from memory by the
          calling instance! Since the Message is given to the Transmitter for further
          processing, and the Transmitter decides when to send. Therefore, the Transmitter
          will also remove the Message from memory.
      \endif

      \if german
        Sendet übergebene Message über seinen Transmitter an den Server.
        \param[in] pTcpMessage Das zu sendende Nachrichtenpacket.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Die übergebene Message darf von der aufrufenden Instanz nicht
          aus dem Speicher entfernt werden! Da die Message zur Weiterverarbeitung
          an den Transmitter gegeben wird, und dieser entscheidet wann gesendet wird.
          Darum wird dieser auch die Message aus dem Speicher entfernen.
      \endif
      */
      virtual int send( Message * pTcpMessage ) override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The transferred Message is inserted into the MsgQueue, usually by the
        server via the ClientReceiver, and thus processed.
        \param[in] pTcpServerMessage The message received from the server.
        \return A return < 0 indicates an error.
      \endif

      \if german
        Es wird die übergebene Message, i.d.R. vom Server über den ClientReceiver
        in die MsgQueue eingefügt und somit verarbeitet.
        \param[in] pTcpServerMessage Die erhaltene Nachricht vom Server.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int acceptServerMessage( Message * pTcpServerMessage );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the hostname as IP address.
        \param[in,out] Socketadress The IP address is supplied in this structure.
        \param[in] rsHostnameOrIp The host name (e.g. "LOCALHOST").
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Liefert den Hostnamen als IP-Adresse.
        \param[in,out] Socketadress In dieser Struktur wird die IP-Adresse geliefert.
        \param[in] rsHostnameOrIp Der Hostname (z.B. "LOCALHOST").
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int getSocketAdrFromString( SOCKADDR_IN * Socketadress,
                                  const std::string & rsHostnameOrIp ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the host name or IP address (for example, "LOCALHOST" or
        "192.168.2.102") from #m_sHostname.
        \return See above.
      \endif

      \if german
        Liefert den Hostname oder IP-Adresse (z.B. "LOCALHOST" oder
        "192.168.2.102") aus #m_sHostname.
        \return s.o.
      \endif
      */
      std::string getHostName() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the IP address and the port.
        \param[in,out] rsIPAdress The IP address (e.g. "192.168.2.102").
        \param[in,out] ruiPort The port which is addressed.
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Liefert die IP-Adresse und den Port.
        \param[in,out] rsIPAdress Die IP-Adresse (z.B. "192.168.2.102").
        \param[in,out] ruiPort Der Port welcher angesprochen wird.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int getPeerName( std::string& rsIPAdress, unsigned int& ruiPort ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the ActionCode of the Message being waited for.
        \return see above.
      \endif

      \if german
        Liefert den ActionCode der Message, auf die gewartet wird.
        \return s.o.
      \endif
      */
      ActionCode getWaitingServerMsgActionCode() const;
      //-----------------------------------------------------------------------


    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Should process the message from the server in subclasses. In the overwritten
        methods, <B>always</B> this method must be called, because among other things
        the passed Message is removed here from the memory.
        \param[in] pTcpMessage The message to be processed from the server.
      \endif

      \if german
        Soll in Unterklassen die Nachricht vom Server verarbeiten. In den
        überschriebenen Methoden mus <B>immer</B> diese Methode aufgerufen werden,
        weil hier u.a. die übergebene Message aus dem Speicher entfernt wird.
        \param[in] pTcpMessage Die zu verarbeitende Nachricht vom Server.
      \endif
      */
      virtual void processServerMessage( Message * pTcpMessage );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Synchronizes the data size with the server.
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Synchronisiert die Datengröße mit dem Server.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      virtual int syncDataSize();
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Client & operator= ( const Client & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( default : 4275 )
  #endif
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
