#ifndef INC_TCPCLIENTRECEIVER_H
#define INC_TCPCLIENTRECEIVER_H
//-----------------------------------------------------------------------------
/*!
\file TcpClientReceiver.h
\brief
\if english
  Header for ClientReceiver.
\endif

\if german
  Header für ClientReceiver.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpDataReceiver.h"

#include "TcpMain.h"

#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  //Forwards
  class AbstractMessage;
  class Client;
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Class that communicates with a Server via a SOCKET and delivers its receive
    result of type Message to its "owner", which must be a Client. You can use
    the method ClientReceiver::acceptMessage(AbstractMessage * pTcpMessage) to
    change the delivery to the "owner" in subclasses by overwriting the method.
  \endif

  \if german
    Klasse, welche über ein SOCKET mit einem Server kommunizieret und ihr
    Empfangsergebnis vom Typ Message, an ihren "Besitzer", welcher ein Client
    sein muss, liefert. Das Liefern an den "Besitzer" kann über Methode
    ClientReceiver::acceptMessage(AbstractMessage * pTcpMessage) in Unterklassen
    durch Überschreiben der Methode abgeändert werden.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll ClientReceiver : public DataReceiver
  {
    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        Auxiliary variable to indicate whether a connection is already established.
      \endif

      \if german
        Hilfsvariable um zu kennzeichnen, ob schon Verbindung aufgebaut ist.
      \endif
      */
      bool m_bHaveConnection;
      //-----------------------------------------------------------------------

    protected:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ClientReceiver();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ClientReceiver( SOCKET );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ClientReceiver( TcpMain *, SOCKET );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ClientReceiver( TcpMain *, SOCKET, unsigned int );
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] pParent The owning parent object.
        \param[in] Socket The socket through which communication takes place.
        \param[in] uiDataSize The package size.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] pParent Das besitzende Elternobjekt.
        \param[in] Socket Der Socket über den kommuniziert wird.
        \param[in] uiDataSize Die Paketgröße.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      ClientReceiver( Client * pParent, SOCKET Socket, unsigned int uiDataSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~ClientReceiver();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ClientReceiver( const ClientReceiver & );
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Accepts streams from its socket (is blocked until received), turns the
        data into a Message which is further processed in 
        ClientReceiver::acceptMessage(AbstractMessage * pTcpMessage).
      \endif

      \if german
        Nimmt von seinem Socket Streams entgegen (ist bis zu Empfang blockiert),
        macht aus den Daten eine Message welche in
        ClientReceiver::acceptMessage(AbstractMessage * pTcpMessage) weiter
        verarbeitet wird.
      \endif
      */
      virtual void work() override;
      //-----------------------------------------------------------------------
      #if defined (__PTHREADS)
      /*
      \if english
        When the thread ends, this virtual method is called.
        \note This method only exists when compiled with POSIX threads (defined
          (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        \note Diese Methode gibt es nur, wenn mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
        virtual void cleanUp();
      #endif
      //-----------------------------------------------------------------------
      /*!
      \if english
        Forwards the transferred message to its Client, which also deletes the
        message after processing. <br>
        The Client can then process these further in subclasses if necessary.
        If processing in a subclass of ClientReceiver is required, this method
        can also be overwritten.
        \param[in] pTcpMessage The received message.
        \return A return < 0 indicates an error.
        \attention The passed object must not be removed from memory by the calling
          instance.
      \endif

      \if german
        Leitet die übergebene Nachricht an seinen Client weiter, der die Nachricht
        nach der Verarbeitung auch löscht. <br>
        Der Client kann diese dann ggf. in Unterklassen weiter verarbeiten. Wenn
        die Bearbeitung in einer Unterklasse von ClientReceiver gewünscht wird,
        kann diese Methode auch überschrieben werden.
        \param[in] pTcpMessage Die empfangene Nachricht.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Das übergebene Objekt darf von der aufrufenden Instanz nicht aus dem
          Speicher entfernt werden.
      \endif
      \sa Message
      */
      virtual int acceptMessage( AbstractMessage * pTcpMessage ) override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Generates a Message from the transferred data buffer and its size.
        \param[in,out] prTcpMessage Pointer to newly generated Message. This
          object takes care of removing the pszTransmitData from the memory.
        \param[in] pszTransmitData The data buffer including all required
          information.
        \param[in] uiTransmitDataSize The size or length of the data buffer to be
          sent.
        \attention The supplied object must be removed from memory by the calling
          instance.
        \exception eut::ErrorNException If no Message can be instantiated from the data
          (because, for example, the header is incorrect), an exception is thrown.
      \endif

      \if german
        Generiert eine Message aus dem übergebenen Datenpuffer und dessen Größe.
        \param[in,out] prTcpMessage Zeiger auf neu generierte Message. Dieses
          Objekt übernimmt das entfernen der pszTransmitData aus dem Speicher.
        \param[in] pszTransmitData Der Datenpuffer inkl. aller benötigten
          Informationen.
        \param[in] uiTransmitDataSize Die Größe bzw. Länge des zu sendenden Datenpuffers.
        \attention Das gelieferte Objekt muss von der aufrufenden Instanz aus dem
          Speicher entfernt werden.
        \exception eut::ErrorNException Wenn aus den Daten keine Message instantiiert werden
          kann (weil z.B. der Header nicht stimmt), wird eine Exception geworfen.
      \endif
      \sa Message
      */
      virtual void generateNewTcpMessage( AbstractMessage *& prTcpMessage,
                                          char * pszTransmitData,
                                          unsigned int uiTransmitDataSize ) const override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Generates a Message from transferred parameters.
        \param[in,out] prTcpMessage Pointer to newly generated Message. This
          object takes over the removal of the pszRawMsg from the memory.
        \param[in] pszRawMsg The message data.
        \param[in] uiRawMsgSize The size or length of the message data.
        \param[in] uiMessageID A unique ID.
        \param[in] iActionCode The ActionCode.
        \param[in] rstrInfo An additional text info
        \attention The supplied object must be removed from memory by the calling instance.
        \exception eut::ErrorNException If no Message can be instantiated from the parameters,
          an exception is thrown.
      \endif

      \if german
        Generiert eine Message aus übergebenen Parametern.
        \param[in,out] prTcpMessage Zeiger auf neu generierte Message. Dieses
          Objekt übernimmt das entfernen der pszRawMsg aus dem Speicher.
        \param[in] pszRawMsg Die Nachrichtendaten.
        \param[in] uiRawMsgSize Die Größe bzw. Länge der Nachrichtendaten.
        \param[in] uiMessageID Eine eindeutige ID.
        \param[in] iActionCode Der ActionCode.
        \param[in] rstrInfo Eine zusätzliche Text-Info
        \attention Das gelieferte Objekt muss von der aufrufenden Instanz aus dem
          Speicher entfernt werden.
        \exception eut::ErrorNException Wenn aus den Parametern keine Message instantiiert
          werden kann, wird eine Exception geworfen.
      \endif
      \sa Message
      */
      virtual void generateNewTcpMessage( AbstractMessage *& prTcpMessage,
                                          char * pszRawMsg,
                                          unsigned int uiRawMsgSize,
                                          unsigned int uiMessageID,
                                          int iActionCode,
                                          const std::string & rstrInfo ) const override;
      //-----------------------------------------------------------------------

    public:
      //========== Methoden ========
      //-----------------------------------------------------------------------
      /*!
      \if english
        Terminates the thread.
      \endif

      \if german
        Beendet den Thread.
      \endif
      */
      #if defined (__PTHREADS)
        virtual int cancel();
      #else
        virtual void cancel() override;
      #endif
      //-----------------------------------------------------------------------
      /*!
      \if english
        Is connected?
        \return If true, the client Receiver has a SOCKET.
      \endif

      \if german
        Ist verbunden?
        \return Wenn true, hat der ClientReceiver einen SOCKET.
      \endif
      */
      bool connected() const;
     //-------------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      ClientReceiver & operator= ( const ClientReceiver & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
