#ifndef INC_TCPDATARECEIVER_H
#define INC_TCPDATARECEIVER_H
//-----------------------------------------------------------------------------
/*!
\file TcpDataReceiver.h
\brief
\if english
  Header for DataReceiver.
\endif

\if german
  Header für DataReceiver.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpReceiver.h"

#include "TcpMain.h"

#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  //Forwards
  class AbstractMessage;
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Superclass for all TcpReceiver that communicate via a SOCKET and deliver their
    receive result, a type derived from AbstractMessage, to their "owner".
    <br>
    Since a message can be composed (it is a stream, which may be delivered in
    chunks), a buffer is created here, which contains data as long as the stream
    is not complete.
  \endif

  \if german
    Oberklasse für alle Receiver, welche über ein SOCKET kommunizieren und ihr
    Empfangsergebnis, ein von AbstractMessage abgeleiteter Typ , an ihren
    "Besitzer" liefert.
    <br>
    Da eine Nachricht zusammengesetzt sein kann (ist ja ein Stream, welcher ggf.
    in Häppchen geliefert wird), wird hier ein Puffer etapliert, welche solange
    Daten enthält, solnge der Stream nicht vollständig ist.

  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll DataReceiver : public Receiver
  {
    protected:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The data packet size.
      \endif

      \if german
        Die Daten-Packetgröße.
      \endif
      */
      unsigned int m_uiDataSize;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The stream size (from the header).
      \endif

      \if german
        Die Streamgröße (aus dem Header).
      \endif
      */
      unsigned int m_uiStreamSize;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The current stream position.
      \endif

      \if german
        Die aktuelle Streamposition.
      \endif
      */
      unsigned int m_uiStreamPosistion;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The received buffer, possibly composed of several sent #m_pszSentBuffer.
      \endif

      \if german
        Der empfangene Buffer, ggf. zusammengestzt aus mehreren gesendeten #m_pszSentBuffer.
      \endif
      \sa m_pszSentBuffer
      */
      char * m_szStreamBuffer;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The received buffer in one receive step. Is transferred to the #m_szStreamBuffer.
        If the size of the expected data packet is not yet reached, the next received
        buffer is appended to #m_szStreamBuffer.
      \endif

      \if german
        Der empfangene Buffer in einem Empfangs-Schritt. Wird in den #m_szStreamBuffer
        übertragen. Wenn die Größe des erwarteten Datenpakets noch nicht erreicht ist,
        wird der nächste empfangene Buffer an #m_szStreamBuffer angehängt.
      \endif
      */
      char * m_pszSentBuffer;
      //-----------------------------------------------------------------------

      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      DataReceiver();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      DataReceiver( SOCKET );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      DataReceiver( TcpMain *, SOCKET );
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] pParent The owning parent object.
        \param[in] Socket The socket through which communication takes place.
        \param[in] uiDataSize The package size.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] pParent Das besitzende Elternobjekt.
        \param[in] Socket Socket über den kommuniziert wird.
        \param[in] uiDataSize Die Paketgröße.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      DataReceiver( TcpMain * pParent, SOCKET Socket, unsigned int uiDataSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor. It cleans up the individual buffers.
      \endif

      \if german
        Destruktor. Es werden die einzelnen Buffer aufgeräumt.
      \endif
      */
      virtual ~DataReceiver();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      DataReceiver( const DataReceiver & );
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the packet size.
        \param[in] uiDataSize The package size.
      \endif

      \if german
        Setzt die Paketgröße.
        \param[in] uiDataSize Die Paketgröße.
      \endif
      */
      void setDataSize( unsigned int uiDataSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the packet size.
        \return, see above.
      \endif

      \if german
        Liefert die Paketgröße.
        \return s.o.
      \endif
      */
      unsigned int getDataSize() const;
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Initializes the stream data (sets everything to zero).
      \endif

      \if german
        Initialisiert die Stream-Daten (setzt alles auf Null).
      \endif
      */
      void initStreamData();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Processes the received data and constructs a message from it (derived from
        AbstractMessage by calling the method generateNewTcpMessage (...) ) and
        passes it on to its owner via acceptMessage (...) .
        \param[in] pszSentBuffer The received buffer
        \param[in] t_uiSentBufferSize The size of the buffer received
        \return A return < 0 indicates an error.
      \endif

      \if german
        Verarbeitet die empfangenen Daten und baut daraus eine Nachricht (abgeleitet
        von AbstractMessage durch Aufruf der Methode generateNewTcpMessage (...) )
        und liefert diese über acceptMessage (...) an seinen Besitzer weiter.
        \param[in] pszSentBuffer Der empfangene Buffer
        \param[in] t_uiSentBufferSize Die Größe des mpfangene Buffers
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int processReceivedData( char * pszSentBuffer, unsigned int t_uiSentBufferSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        In subclasses, forwards the passed message to its "owner" or edits it
        itself. The message must be deleted in any case.
        \return A return < 0 indicates an error.
        \attention The passed object must not be removed from memory by the calling
          instance.
      \endif

      \if german
        Leitet in Unterklassen die übergebene Nachricht an seinen "Besitzer" weiter
        oder bearbeitet diese selbst. Die Nachricht muss in jedem Fall gelöschet
        werden.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Das übergebene Objekt darf von der aufrufenden Instanz nicht aus dem
          Speicher entfernt werden.
      \endif
      \sa AbstractMessage, Message, ServerAcceptMessage, ServerMessage
      */
      virtual int acceptMessage( AbstractMessage * ) = 0;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Generates in subclasses a message derived from AbstractMessage from the
        passed data buffer and its size.
        \param[in,out] prTcpAbstractMessage Pointer to newly generated Message.
          This object takes care of removing the pszTransmitData from the memory.
        \param[in] pszTransmitData The data buffer including all required
          information.
        \param[in] uiTransmitDataSize The size or length of the data buffer to be
          sent.
        \attention The supplied object must be removed from memory by the calling
          instance.
      \endif

      \if german
        Generiert in Unterklassen eine von AbstractMessage abgeleitete Nachricht
        aus dem übergebenen Datenpuffer und dessen Größe.
        \param[in,out] prTcpAbstractMessage Zeiger auf neu generierte Nachricht.
          Dieses Objekt übernimmt das entfernen der pszTransmitData aus dem Speicher.
        \param[in] pszTransmitData Der Datenpuffer inkl. aller benötigten
          Informationen.
        \param[in] uiTransmitDataSize Die Größe bzw. Länge des zu sendenden Datenpuffers.
        \attention Das gelieferte Objekt muss von der aufrufenden Instanz aus dem
          Speicher entfernt werden.
      \endif
      \sa AbstractMessage, Message, ServerAcceptMessage, ServerMessage
      */
      virtual void generateNewTcpMessage( AbstractMessage *& prTcpAbstractMessage,
                                          char * pszTransmitData,
                                          unsigned int uiTransmitDataSize ) const = 0;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Generates in subclasses a message derived from AbstractMessage from
        passed parameters.
        \param[in,out] prTcpAbstractMessage Pointer to newly generated Message.
          This object takes over the removal of the pszRawMsg from the memory.
        \param[in] pszRawMsg The message data.
        \param[in] uiRawMsgSize The size or length of the message data.
        \param[in] uiMessageID A unique ID.
        \param[in] iActionCode The ActionCode.
        \param[in] rstrInfo An additional text info
        \attention The supplied object must be removed from memory by the calling
          instance.
      \endif

      \if german
        Generiert in Unterklassen eine von AbstractMessage abgeleitete Nachricht
        aus übergebenen Parametern.
        \param[in,out] prTcpAbstractMessage Zeiger auf neu generierte Nachricht.
          Dieses Objekt übernimmt das entfernen der pszRawMsg aus dem Speicher.
        \param[in] pszRawMsg Die Nachrichtendaten.
        \param[in] uiRawMsgSize Die Größe bzw. Länge der Nachrichtendaten.
        \param[in] uiMessageID Eine eindeutige ID.
        \param[in] iActionCode Der ActionCode.
        \param[in] rstrInfo Eine zusätzliche Text-Info
        \attention Das gelieferte Objekt muss von der aufrufenden Instanz aus dem
          Speicher entfernt werden.
      \endif
      \sa AbstractMessage, Message, ServerAcceptMessage, ServerMessage
      */
      virtual void generateNewTcpMessage( AbstractMessage *& prTcpAbstractMessage,
                                          char * pszRawMsg,
                                          unsigned int uiRawMsgSize,
                                          unsigned int uiMessageID,
                                          int iActionCode,
                                          const std::string & rstrInfo ) const = 0;
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      DataReceiver & operator= ( const DataReceiver & );
      //-----------------------------------------------------------------------
  };
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
