#ifndef INC_TCPMAIN_H
#define INC_TCPMAIN_H
//-----------------------------------------------------------------------------
/*!
\file TcpMain.h
\brief
\if english
  Header for TcpMain
\endif

\if german
  Header für TcpMain
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"

#if defined (__PTHREADS)
  #include "EuEndlessPThread.h"
  #include <semaphore.h>
#else
  #include "EuRepeatingThread.h"
#endif

#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#elif defined(__GNUC__)
  /*
  #include <stdio.h>
  #include <stdlib.h>
  #include <unistd.h>
  */
  #include <string.h>

  #include <errno.h>
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <netdb.h>
  #include <arpa/inet.h>
  #define SOCKET uintptr_t
  #define SOCKADDR_IN sockaddr_in
  #define SOCKADDR sockaddr
  #define HOSTENT hostent
  #define INVALID_SOCKET  (SOCKET)(~0)
  #define SOCKET_ERROR            (-1)
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  // Forwards
  class Message;
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( disable : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( disable : 4275 )
  #endif
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Superclass for Client and Server, which 'learns' its know-how in the
    derived classes.
  \endif

  \if german
    Oberklasse für Client sowie Server, welche ihr KnowHow in den
    abgeleiteten Klassen 'lernt'.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  #if defined (__PTHREADS)
    class __tcp_export_dll TcpMain : public EuEndlessPThread
  #else
    class __tcp_export_dll TcpMain : public eut::RepeatingThread
  #endif
  {
    private:
      //============== static-attributes
      //-----------------------------------------------------------------------
      /*!
      \if english
        This counter is incremented each time an object of this class or subclass
        is instantiated. With each call to setWSACleanup() it is decremented again.
      \endif

      \if german
        Bei jeder Instanziierung eines Objektes dieser Klasse bzw. Unterklasse wird
        dieser Zähler inkrementiert . Bei jedem Aufruf von setWSACleanup()  wiederum
        dekrementieret.
      \endif
      \sa setWSACleanup()
      */
      static unsigned int m_uiTcpMainCounter;
      //-----------------------------------------------------------------------

    protected:
      //============== attributes
      //-----------------------------------------------------------------------
      /*!
      \if english
        Waiting time for connection responses. if not set, the wait may be eternal!
      \endif

      \if german
        Wartezeit für Verbindungsantworten. wenn nicht gesetzt, wird ggf. ewig
        gewartet!
      \endif
      */
      int m_iSecTimeOut;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The port of the socket. <br>
        <i>Further information under <a href="https://en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers" target="_blank">en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers</a></i>
      \endif

      \if german
        Der Port des Sockets. <br>
        <i>Weitere Infos unter <a href="https://de.wikipedia.org/wiki/Liste_der_standardisierten_Ports" target="_blank">de.wikipedia.org/wiki/Liste_der_standardisierten_Ports</a></i>
      \endif
      */
      unsigned int m_uiServerPort;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Packets are made from the data stream; here the packet size (default is
        ::DEFAULT_DATASIZE).
      \endif

      \if german
        Es werden aus dem Datenstrom Packete gemacht; hier die Paketgröße (default
        ist ::DEFAULT_DATASIZE).
      \endif
      */
      unsigned int m_uiDataSize;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The Socket AddressFamily (usually AF_INET) to instantiate a socket.
      \endif

      \if german
        Die Socket-AdressFamily (i.d.R. AF_INET) um einen Socket zu instanziieren.
      \endif
      */
      int m_iAdressFamily;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The socket type (usually SOCK_STREAM) to instantiate a socket.
      \endif

      \if german
        Der Socket-Typ (i.d.R. SOCK_STREAM) um einen Socket zu instanziieren.
      \endif
      */
      int m_iSocketType;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The socket protocol (usually 0) to instantiate a socket.
      \endif

      \if german
        Das Socket-Protokol (i.d.R. 0) um einen Socket zu instanziieren.
      \endif
      */
      int m_iProtocol;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The socket.
      \endif

      \if german
        Der/die/das ? Socket.
      \endif
      */
      SOCKET m_MainSocket;
      //-----------------------------------------------------------------------

      //============== ctor
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      TcpMain();
      //-----------------------------------------------------------------------

    public:
      //============== ctors
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] uiPort The port of the server.
        \param[in] iAdressFamily The socket address family to instantiate a socket
          (usually AF_INET).
        \param[in] iSocketType The socket type (usually SOCK_STREAM).
        \param[in] iProtocol The socket protocol (usually 0).
        \param[in] iSecTimeOut The time until waiting for a connection or waiting
          for data is aborted.

          It is started under Windows WSA.

        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] uiPort Der Port des Servers.
        \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
          instanziieren (i.d.R. AF_INET).
        \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM).
        \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
        \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
           warten auf Daten abgebrochen wird.

        Es wird unter Windows WSA gestartet.

        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      TcpMain(  unsigned int uiPort,
                int iAdressFamily,
                int iSocketType,
                int iProtocol,
                int iSecTimeOut );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] uiPort The port of the server.
        \param[in] iAdressFamily The socket address family to instantiate a socket
          (usually AF_INET).
        \param[in] iSocketType The socket type (usually SOCK_STREAM).
        \param[in] iProtocol The socket protocol (usually 0).
        \param[in] uiDataSize The packet size.
        \param[in] iSecTimeOut The time until waiting for a connection or waiting
          for data is aborted.

          It is started under Windows WSA.

        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] uiPort Der Port des Servers.
        \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
          instanziieren (i.d.R. AF_INET).
        \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM).
        \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
        \param[in] uiDataSize Die Paketgröße muss größer 0 sein.
        \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
          Daten abgebrochen wird.

        Es wird unter Windows WSA gestartet.

        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      TcpMain(  unsigned int uiPort,
                int iAdressFamily,
                int iSocketType,
                int iProtocol,
                unsigned int uiDataSize,
                int iSecTimeOut );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor. It is terminated under Windows WSA.
      \endif

      \if german
        Destruktor. Es wird unter Windows WSA beendet.
      \endif
      */
      virtual ~TcpMain();
      //-----------------------------------------------------------------------

    private:
      //============== copy-ctor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      TcpMain( const TcpMain & );
      //-----------------------------------------------------------------------

    public:
      //============== methods
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the port number.
      \endif

      \if german
        Liefert die Portnummer.
      \endif
      */
      unsigned int getPortNum() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the packet size.
        \param[in] uiDataSize The package size must be greater than 0.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Setzt die Paketgröße.
        \param[in] uiDataSize Die Paketgröße muß gröser 0 sein.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      virtual void setDataSize( unsigned int uiDataSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the packet size.
      \endif

      \if german
        Liefert die Paketgröße.
      \endif
      */
      unsigned int getDataSize() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the socket.
      \endif

      \if german
        Liefert den Socket.
      \endif
      */
      SOCKET getMainSocket() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sends transferred Message to the counterpart.

        If This is a client, the counterpart is the server. If This is a server,
        the counterpart is the server. Sends transferred message to the counterpart
        socket referenced there.
      
        The implementation is executed in the subclass.
        \param[in] pTcpMessage The message to be sent.
        \return A return < 0 indicates an error.
      \endif

      \if german
        Sendet übergebene Message an den Konterpart.

        Wenn This ein Client ist, ist der Konterpart der Server. Wenn This ein
        Server, ist der Konterpart der Server. Es wird übergebene Nachricht an
        den dort referenzierten Konterpart-Socket gesendet.

        Die Implementierung ist in den Unterklasse ausgeführt.
        \param[in] pTcpMessage Die zu versendende Nachricht.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      virtual int send( Message * pTcpMessage ) = 0;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the seconds to wait for responses.
        \param[in] iSecTimeOut The maximum waiting time.
      \endif

      \if german
        Setzt die Sekunden, die auf Antworten geartet wird.
        \param[in] iSecTimeOut Die maximale Wartezeit.
      \endif
      */
      void setTimeOutSec( int iSecTimeOut );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the seconds how long the system waits for answers.
        \return The maximum waiting time.
      \endif

      \if german
        Liefert die Sekunden, wie lange auf Antworten gewartet wird.
        \return Die maximale Wartezeit.
      \endif
      */
      int TimeOutSec();
      //-----------------------------------------------------------------------

    protected:
      //============== methods
      //-----------------------------------------------------------------------
      /*!
      \if english
        It decrements #m_uiTcpMainCounter and on Windows if 0, calls WSACleanup()
        (closes ALL connections)
      \endif

      \if german
        Es wird #m_uiTcpMainCounter dekrementiert und unter Windows wenn 0, WSACleanup()
        aufgerufen (schließt ALLE Verbindungen).
      \endif
      */
      void setWSACleanup() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        It is started under Windows WSA using WSAStartup().
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Es wird unter Windows WSA mittels WSAStartup() gestartet.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int startWinsock() const;
      //-----------------------------------------------------------------------

    private:
      //============== operators
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      TcpMain & operator= ( const TcpMain & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( default : 4275 )
  #endif
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
