#ifndef INC_TCPMESSAGE_H
#define INC_TCPMESSAGE_H
//-----------------------------------------------------------------------------
/*!
\file TcpMessage.h
\brief
\if english
  Header for AbstractMessage, Message, ServerAcceptMessage and ServerMessage.
\endif

\if german
  Header für AbstractMessage, Message, ServerAcceptMessage und ServerMessage.
\endif

\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include <cstring>
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#elif defined(__GNUC__)
  /*
  #include <stdio.h>
  #include <stdlib.h>
  #include <unistd.h>
  */
  #include <string.h>

  #include <errno.h>
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <netdb.h>
  #include <arpa/inet.h>
  #define SOCKET uintptr_t
  #define SOCKADDR_IN sockaddr_in
  #define SOCKADDR sockaddr
  #define HOSTENT hostent
  #define INVALID_SOCKET  (SOCKET)(~0)
  #define SOCKET_ERROR            (-1)
#endif
//-----------------------------------------------------------------------------
#ifdef _MSC_VER
  /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
  #pragma warning( disable : 4251 )
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>> AbstractMessage >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Base class for messages which can be sent, received and processed by the
    Tcp-classes.
  \endif

  \if german
    Basisklasse für Nachrichten, welche von den Tcp-Klassen verschickt,
    empfangen und bearbeitet werden können.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll AbstractMessage
  {
    protected:
      //============== Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      AbstractMessage();
      //-----------------------------------------------------------------------

    public:
      //============== Destruktore
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~AbstractMessage();
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<< AbstractMessage <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>>>> Message >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Contains a data stream which can be sent and received by the Tcp classes.

    The message data is preceded by a header consisting of
      - an abbreviation "hja-tcp" (0-terminated); a simple, non-secure way to filter
        foreign messages.
      - the size of the entire message, i.e. header plus message data (10 digits,
        0-terminated), see #m_nTransmitDataSize
      - a message ID (0-terminated), see #m_uiMessageID
      - a ActionCode (0-terminated), see #m_iActionCode
      - an info (0-terminated), see #m_strInfo
    \endif

  \if german
    Beinhaltet einen Datenstrom, welcher von den Tcp-Klassen versendet und empfangen
    werden kann.

    Den Nachrichtendaten ist ein Header vorangestellt bestehend aus:
      - einem Kürzel "hja-tcp" (0-terminiert); eine einfache, nicht sichere Möglichkeit,
        um fremde Nachrichten zu filtern.
      - die Größe der gesamten Nachricht, also Header plus Nachrichtendaten (10 Stellen,
        0-terminiert), siehe #m_nTransmitDataSize
      - eine Nachrichts-ID (0-terminiert), siehe #m_uiMessageID
      - ein ActionCode (0-terminiert), siehe #m_iActionCode
      - eine Info (0-terminiert), siehe #m_strInfo
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll Message : public virtual AbstractMessage
  {
    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        A unique ID.
      \endif

      \if german
        Eine eindeutige ID.
      \endif
      */
      unsigned int m_uiMessageID;
      //-----------------------------------------------------------------------
      /*!
      \if english
        A code to signal a certain request if necessary.
      \endif

      \if german
        Ein Code um ggf. eine bestimmte Anforderung zu signalisieren.
      \endif
      \sa ActionCode

      \if english
        \attention All numbers < 0 are reserved for error messages!
      \endif

      \if german
        \attention Alle Zahlen < 0 sind reserviert für Fehlermeldungen!
      \endif
      \sa ERRORS
      */
      int m_iActionCode;
      //-----------------------------------------------------------------------
      /*!
      \if english
        A freely usable info.
      \endif

      \if german
        Eine frei nutzbare Info.
      \endif
      */
      std::string m_strInfo;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The size or length of the message data.
      \endif

      \if german
        Die Größe bzw. Länge der Nachrichtsdaten.
      \endif
      */
      std::size_t m_nRawMessageSize;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The message data.
      \endif

      \if german
        Die Nachrichtsdaten.
      \endif
      */
      char * m_pszRawMessage;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The size or length of the send data (header plus message data).
      \endif

      \if german
        Die Größe bzw. Länge der Sendedaten (Header plus Nachrichtsdaten).
      \endif
      */
      std::size_t m_nTransmitDataSize;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The complete send data of the message including header.
      \endif

      \if german
        Die gesamten Sendedaten der Message inkl. Header.
      \endif
      */
      char * m_pszTransmitData;
      //-----------------------------------------------------------------------

    protected:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Message();
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        A constructor that expects a data buffer and its size.

        \param[in] pszTransmitData The data buffer to be transmitted including all
          required Information:
          - the abbreviation "hja-tcp" (0-terminated) a simple non-secure way to
            filter foreign messages.
          - the size of the entire message, i.e. header plus message data
            (0-terminated)
          - A message ID (0-terminated)
          - a ActionCode (0-terminated)
          - an info (0-terminated)
          - the message data of the message

        \param[in] nTransmitDataSize The size or length of the data buffer to be
          sent.
        \attention This object takes over the removal of the pszTransmitData from
          the memory.
        \exception eut::ErrorNException An exception is thrown if the attributes cannot be set
          from the data (e.g. because the header is incorrect).
      \endif

      \if german
        Ein Konstruktor der einen Datenpuffer und dessen Größe erwartet.

        \param[in] pszTransmitData Der zu sendende Datenpuffer inkl. aller benötigten
          Informationen:
          - das Kürzel "hja-tcp" (0-terminiert) eine einfache nicht sichere Möglichkeit,
            um fremde Nachrichten zu filtern.
          - die Größe der gesamten Nachricht, also Header plus Nachrichtendaten
            (0-terminiert)
          - eine Nachrichts-ID (0-terminiert)
          - ein ActionCode (0-terminiert)
          - eine Info (0-terminiert)
          - die Nachrichtendaten der Message

        \param[in] nTransmitDataSize Die Größe bzw. Länge des zu sendenden Datenpuffers.
        \attention Dieses Objekt übernimmt das entfernen der pszTransmitData aus dem
          Speicher.
        \exception eut::ErrorNException Wenn aus den Daten die Attribute nicht gesetzt werden können
          (weil z.B. der Header nicht stimmt), wird eine Exception geworfen.
      \endif
      */
      Message( char * pszTransmitData, std::size_t nTransmitDataSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        A constructor that can build a message to be sent from the transferred
        data.

        \param[in] pszRawMsg The message data.
        \param[in] nRawMsgSize The size or length of the message data.
        \param[in] uiMessageID A unique ID.
        \param[in] iActionCode The ActionCode, see #m_iActionCode.
        \param[in] rstrInfo An additional text info
        \attention This object removes the pszRawMsg from the memory
        \exception eut::ErrorNException If #m_pszTransmitData cannot be set from the data (e.g.
          because attributes are invalid), an exception is thrown.
      \endif

      \if german
        Ein Konstruktor, der aus übergebenen Daten, eine zu sendende Message bauen kann.

        \param[in] pszRawMsg Die Nachrichtendaten.
        \param[in] nRawMsgSize Die Größe bzw. Länge der Nachrichtendaten.
        \param[in] uiMessageID Eine eindeutige ID.
        \param[in] iActionCode Der ActionCode, siehe #m_iActionCode.
        \param[in] rstrInfo Eine zusätzliche Text-Info
        \attention Dieses Objekt übernimmt das entfernen der pszRawMsg aus dem
          Speicher
        \exception eut::ErrorNException Wenn aus den Daten #m_pszTransmitData nicht gesetzt werden
          kann (weil z.B. Attribute ungültig sind), wird eine Exception geworfen.
      \endif
      */
      Message(  char * pszRawMsg,
                std::size_t nRawMsgSize,
                unsigned int uiMessageID,
                int iActionCode,
                const std::string & rstrInfo );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~Message();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Message( const Message & );
      //-----------------------------------------------------------------------

      //+ ============ Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets its attributes from m_pszTransmitData. Is used by
        Message::Message( char *, std::size_t ).
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Setzt seine Attribute aus m_pszTransmitData. Wird von
        Message::Message( char *, std::size_t )
        genutzt.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int initDataFromSendMsg();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Builds #m_pszTransmitData from its attributes. Is used by
        Message( char *, std::size_t, unsigned int, int, const std::string & ).
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Baut #m_pszTransmitData aus seinen Attributen. Wird von
        Message( char *, std::size_t, unsigned int, int, const std::string & )
        genutzt.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int initSendMsgFromData();
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the message data.
      \endif

      \if german
        Liefert die Nachrichtsdaten.
      \endif
      */
      char * getRawMsgPt();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the size of the message data.
      \endif

      \if german
        Liefert die Größe der Nachrichtsdaten.
      \endif
      */
      std::size_t getRawMsgSize() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the transmission data.
      \endif

      \if german
        Liefert die Sendedaten.
      \endif
      */
      char * getTransmitDataPt();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the size of the send data.
      \endif

      \if german
        Liefert die Größe der Sendedaten.
      \endif
      */
      std::size_t getTransmitDataSize() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the unique ID.
      \endif

      \if german
        Liefert die eindeutige ID.
      \endif
      */
      unsigned int getMsgID() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the action code.
        \sa ActionCode and ERRORS
      \endif

      \if german
        Liefert den Aktionscode.
        \sa ActionCode und ERRORS
      \endif
      */
      int getActionCode() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Liefert den Informationstext.
      \endif

      \if german
        Liefert den Informationstext.
      \endif
      */
      std::string getInfo() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the message data.
        \param[in,out] prszRawMsg Here the message data is delivered.
        \param[in] bCopy If set to true, memory is requested by this method for
          passed prszRawMsg, which must be released by the calling instance.
        \return The return value indicates the data size.
        \exception eut::ErrorNException An exception is thrown if no memory could be requested.
      \endif

      \if german
        Liefert die Nachrichtendaten.
        \param[in,out] prszRawMsg Hier werden die Nachrichtendaten geliefert.
        \param[in] bCopy Wenn auf true gesetzt ist, wird von dieser Methode für
          übergebene prszRawMsg Speicher angefordert, welcher von der aufrufenden
          Instanz wieder freigegeben werden muß.
        \return Der Rückgabewert zeigt die Datengröße an.
        \exception eut::ErrorNException Es wird eine Exception geworfen, wenn kein Speicher
          angefordert werden konnte.
      \endif
      */
      std::size_t getRawMsg( char *& prszRawMsg, bool bCopy = true );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the transmission data.
        \param[in,out] prszSendMsg Here the send data is delivered.
        \param[in] bCopy If set to true, memory is requested by this method for passed prszSendMsg, which must be released by the calling instance.
        \return The return value indicates the data size.
        \exception eut::ErrorNException An exception is thrown if no memory could be requested.
      \endif

      \if german
        Liefert die Sendedaten.
        \param[in,out] prszSendMsg Hier werden die Sendedaten geliefert.
        \param[in] bCopy Wenn auf true gesetzt ist, wird von dieser Methode für
          übergebene prszSendMsg Speicher angefordert, welcher von der aufrufenden
          Instanz wieder freigegeben werden muß.
        \return Der Rückgabewert zeigt die Datengröße an.
        \exception eut::ErrorNException Es wird eine Exception geworfen, wenn kein Speicher
          angefordert werden konnte.
      \endif
      */
      std::size_t getTransmitData( char *& prszSendMsg, bool bCopy = true );
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Message & operator= ( const Message & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<<<< Message <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>> ServerAcceptMessage >>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    A special Tcp message, with which the ServerAcceptReceiver can register a new
    Client.
  \endif

  \if german
    Ein spezielle Tcp-Message, damit kann der ServerAcceptReceiver einen neuen
    Client anmelden.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll ServerAcceptMessage : public virtual AbstractMessage
  {
    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The new client socket.
      \endif

      \if german
        Der neue Client-Socket.
      \endif
      */
      SOCKET m_ClientSocket;
      //-----------------------------------------------------------------------

    protected:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      ServerAcceptMessage ();
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] NewSocket The new client socket.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] NewSocket Der neue Client-Socket.
      \endif
      */
      ServerAcceptMessage( SOCKET NewSocket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~ServerAcceptMessage();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerAcceptMessage( const ServerAcceptMessage & );
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the client socket.
      \endif

      \if german
        Liefert den Client-Socket.
      \endif
      */
      SOCKET getSocket();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Closes the client socket.
      \endif

      \if german
        Schließt den Client-Socket.
      \endif
      */
      void closeSocket();
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      ServerAcceptMessage & operator= ( const ServerAcceptMessage & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<< ServerAcceptMessage <<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>>>> ServerMessage >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Is a special Tcp message for the Server. The socket is included so that the
    server knows to whom it should send the data.
  \endif

  \if german
    Ist eine spezielle Tcp-Message für den Server. Damit dieser weiß, an
    wen er die Daten senden soll, ist der Socket enthalten.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll ServerMessage : public Message, public ServerAcceptMessage
  {
    protected:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      ServerMessage();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      ServerMessage( char *, std::size_t );
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Art constructor with parameters.
        \param[in] ClientSocket The client socket.
        \param[in] pszTransmitData The data buffer to be transmitted including all
          required Information:
          - the abbreviation "hja-tcp" (0-terminated) a simple non-secure way to
            filter foreign messages.
          - the size of the entire message, i.e. header plus message data
            (0-terminated)
          - A message ID (0-terminated)
          - a ActionCode (0-terminated)
          - an info (0-terminated)
          - the message data of the message

        \param[in] nTransmitDataSize The size or length of the data buffer to be
          sent.
        \attention This object takes over the removal of the pszTransmitData from
          the memory.
        \exception eut::ErrorNException An exception is thrown if the attributes cannot be set
          from the data (e.g. because the header is incorrect).
      \endif

      \if german
        Kunstruktor mit Parametern.
        \param[in] ClientSocket Der Client-Socket.
        \param[in] pszTransmitData Der zu sendende Datenpuffer inkl. aller benötigten
          Informationen:
          - das Kürzel "hja-tcp" (0-terminiert) eine einfache nicht sichere Möglichkeit,
            um fremde Nachrichten zu filtern.
          - die Größe der gesamten Nachricht, also Header plus Nachrichtendaten
            (0-terminiert)
          - eine Nachrichts-ID (0-terminiert)
          - ein ActionCode (0-terminiert)
          - eine Info (0-terminiert)
          - die Nachrichtendaten der Message

        \param[in] nTransmitDataSize Die Größe bzw. Länge des zu sendenden Datenpuffers.
        \attention Dieses Objekt übernimmt das entfernen der pszTransmitData aus dem
          Speicher.
        \exception eut::ErrorNException Wenn aus den Daten die Attribute nicht gesetzt werden können
          (weil z.B. der Header nicht stimmt), wird eine Exception geworfen.
      \endif
      */
      ServerMessage(  SOCKET ClientSocket,
                      char * pszTransmitData,
                      std::size_t nTransmitDataSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Art constructor with parameters.
        \param[in] ClientSocket The client socket.
        \param[in] pszRawMsg The message data.
        \param[in] nRawMsgSize The size or length of the message data.
        \param[in] uiMessageID A unique ID.
        \param[in] iActionCode The ActionCode, see #m_iActionCode.
        \param[in] rstrInfo An additional text info
        \attention This object removes the pszRawMsg from the memory
        \exception eut::ErrorNException If #m_pszTransmitData cannot be set from the data (e.g.
          because attributes are invalid), an exception is thrown.
      \endif

      \if german
        Kunstruktor mit Parametern.
        \param[in] ClientSocket Der Client-Socket.
        \param[in] pszRawMsg Die Nachrichtendaten.
        \param[in] nRawMsgSize Die Größe bzw. Länge der Nachrichtendaten.
        \param[in] uiMessageID Eine eindeutige ID.
        \param[in] iActionCode Der ActionCode, siehe #m_iActionCode.
        \param[in] rstrInfo Eine zusätzliche Text-Info
        \attention Dieses Objekt übernimmt das entfernen der pszRawMsg aus dem
          Speicher
        \exception eut::ErrorNException Wenn aus den Daten #m_pszTransmitData nicht gesetzt werden
          kann (weil z.B. Attribute ungültig sind), wird eine Exception geworfen.
      \endif
      */
      ServerMessage(  SOCKET ClientSocket,
                      char * pszRawMsg,
                      std::size_t nRawMsgSize,
                      unsigned int uiMessageID,
                      int iActionCode,
                      const std::string & rstrInfo);
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~ServerMessage();
      //-----------------------------------------------------------------------
    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerMessage( const ServerMessage & );

      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      ServerMessage & operator= ( const ServerMessage & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<<< ServerMessage <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
  #endif
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
