#ifndef INC_TCPRECEIVERTRANSMITTER_H
#define INC_TCPRECEIVERTRANSMITTER_H
//-----------------------------------------------------------------------------
/*!
\file TcpReceiverTransmitter.h
\brief
\if english
  Header for ReceiverTransmitter.
\endif

\if german
  Header für ReceiverTransmitter.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"

#if defined (__PTHREADS)
  #include "EuEndlessPThread.h"
#else
  #include "EuRepeatingThread.h"
#endif

#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#elif defined(__GNUC__)
  /*
  #include <stdio.h>
  #include <stdlib.h>
  #include <unistd.h>
  */
  #include <string.h>

  #include <errno.h>
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <netdb.h>
  #include <arpa/inet.h>
  #define SOCKET uintptr_t
  #define SOCKADDR_IN sockaddr_in
  #define SOCKADDR sockaddr
  #define HOSTENT hostent
  #define INVALID_SOCKET  (SOCKET)(~0)
  #define SOCKET_ERROR            (-1)
#endif
//-----------------------------------------------------------------------------
#ifdef _MSC_VER
  /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
  #pragma warning( disable : 4251 )
  /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
  #pragma warning( disable : 4275 )
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Superclass for all Receiver and Transmitter, which communicate via a SOCKET.
  \endif

  \if german
    Oberklasse für alle Receiver und Transmitter, welche über ein SOCKET kommunizieren.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  #if defined (__PTHREADS)
    class __tcp_export_dll ReceiverTransmitter : public EuEndlessPThread
  #else
    class __tcp_export_dll ReceiverTransmitter : public eut::RepeatingThread
  #endif
  {
    protected:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        All derived classes communicate via a socket.
      \endif

      \if german
        Alle abgeleiteten Klassen kommunizieren über einen Socket.
      \endif
      */
      SOCKET m_Socket;
      //-----------------------------------------------------------------------

      //============== Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      ReceiverTransmitter();
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] Socket The socket via which communication takes place.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] Socket Socket über den kommuniziert wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      ReceiverTransmitter( SOCKET Socket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor. The socked is closed.
      \endif

      \if german
        Destruktor. Es wird der Socked geschlossen.
      \endif
      */
      virtual ~ReceiverTransmitter();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ReceiverTransmitter( const ReceiverTransmitter &  );
      //-----------------------------------------------------------------------

    public:
      //========== Methoden ========
      //-----------------------------------------------------------------------
      /*!
      \if english
        These threads must not pause because they themselves are blocking!
        \exception eut::ErrorNException An exception is thrown when called.
      \endif

      \if german
        Diese Threads dürfen nicht pausieren, da sie selbst blockieren!
        \exception eut::ErrorNException Bei einem Aufruf wird eine Exception geworfen.
      \endif
      */
      #if defined (__PTHREADS)
        virtual int takeABreak();
      #else
        virtual void takeABreak() override;
      #endif
      //-----------------------------------------------------------------------
      /*!
      \if english
        These threads must not pause because they themselves are blocking!
        \exception eut::ErrorNException An exception is thrown when called.
      \endif

      \if german
        Diese Threads dürfen nicht pausieren, da sie selbst blockieren!
        \exception eut::ErrorNException Bei einem Aufruf wird eine Exception geworfen.
      \endif
      */
      #if defined (__PTHREADS)
        virtual int finishBreak();
      #else
        virtual void finishBreak() override;
      #endif
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the IP address and the port.
        \param[in,out] rstrIPAdress The IP address (e.g. "LOCALHOST" or "192.168.2.102").
        \param[in,out] ruiPort The port which is addressed.
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Liefert die IP-Adresse und den Port.
        \param[in,out] rstrIPAdress Die IP-Adresse (z.B. "LOCALHOST" oder "192.168.2.102").
        \param[in,out] ruiPort Der Port welcher angesprochen wird.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int getPeerName( std::string & rstrIPAdress, unsigned int & ruiPort ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the socket.
      \endif

      \if german
        Liefert das Socket.
      \endif
      */
      SOCKET Socket() const;
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      ReceiverTransmitter & operator= ( const ReceiverTransmitter & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( default : 4275 )
  #endif
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
