#ifndef INC_TCPSERVER_H
#define INC_TCPSERVER_H
//-----------------------------------------------------------------------------
/*!
\file TcpServer.h
\brief
\if english
  Header for Server and Server::ServerReceiverTransmitter
\endif

\if german
  Header für Server und Server::ServerReceiverTransmitter
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpMain.h"
#include "TcpMessage.h"
#include <queue>
#include <map>
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( disable : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( disable : 4275 )
  #endif
  //---------------------------------------------------------------------------
  //Forwards
  class Transmitter;
  class ServerReceiver;
  class ServerAcceptReceiver;
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    A simple TCP server class, which should learn its know-how in the derived classes.

    Server is a thread that allows Tcp Client's to connect to it.

    TcpServers have as superclass attributes the port, the data size which is sent,
    the address family, the socket type, the protocol type and a main socket (here
    AcceptSocket).

    In addition, the following attributes are available here: a ServerAcceptReceiver,
    which processes the new requests as a thread, a MsgQueue, which records the
    individual messages of the individual ClientSockets, a counter that contains the
    maximum number of clients and a ClientMap with the individual ServerReceiverTransmitter
    objects.

    For each accepted client, an object of the embedded class ServerReceiverTransmitter
    is instantiated, which has a Transmitter and a Receiver, and stored in the
    ClientMap. Transmitter and Receiver are also threads.

    Starting the server:
    ------------------
    During instantiation with transfer of port, address family, socket type,
    protocol, data size and maximum number of clients, the own thread and the
    ServerAcceptReceiver is initialized (with the calling server as parent object
    and the main socket) and also started as a thread. This always blocks until a
    client logs on.

    New client wants to connect:
    ---------------------------------
    When a client logs on to the ServerAcceptReceiver, a new ServerAcceptMessage
    instantiated (with the new SOCKET of the client as parameter) and to the server
    via Server::acceptClientMessage(AbstractMessage *) passed on.

    In Server::acceptClientMessage(AbstractMessage *) the passed
    ServerAcceptMessage is appended to the MsgQueue. When the MsgQueue is
    processed in Server::work(), the ServerAcceptMessage is found.

    If the maximum number of clients is exceeded, the client is informed and the
    client socket is closed.

    If the client socket is not closed, a ServerReceiverTransmitter is instantiated using
    a virtual method Server::generateNewServerReceiverTransmitter (...). The server is
    passed as the parent object for the ServerReceiver that is still to be
    instantiated, the new socket and the data size. A Transmitter (which sends data to
    the new client) and a ServerReceiver (which receives data from the new client)
    are instantiated for the ServerReceiverTransmitter. It is possible to establish a
    derived ServerReceiverTransmitter with derived Transmitter or ServerReceiver.

    The new ServerReceiverTransmitter is inserted into the ClientMap.

    Now the server method Server::processClientMessage(AbstractMessage *) is
    called, in which further processing can be carried out in subclasses. Among other
    things, the ServerAcceptMessage is removed from the memory.

    The new ServerReceiverTransmitter or its ServerReceiver in the ClientMap is now
    responsible for further messages from the client.

    Processing data from the client:
    -----------------------------
    The ServerReceiver waits in its thread method ServerReceiver::work() for
    messages from the connected client. If the client sends (abort or data), the
    ServerReceiver creates a new ServerMessage using the method
    ServerReceiver::generateNewTcpMessage (... ) (can be overwritten in subclasses
    to return derived message types) and forwarded to the parent server
    Server::acceptClientMessage(AbstractMessage *) using the method
    ServerReceiver::acceptMessage(AbstractMessage *).

    In subclasses, a derived ServerReceiver can take over processing in the
    overwritten ServerReceiver::acceptMessage(AbstractMessage *) and/or the
    derived Server can take over processing in the overwritten
    Server::acceptClientMessage(AbstractMessage *).

    In the method Server::acceptClientMessage(AbstractMessage *), the
    transferred ServerMessage is appended to the MsgQueue. When the MsgQueue is
    processed, the ServerMessage is found and forwarded to
    Server::processClientMessage(AbstractMessage *). If the client has broken
    the connection, the message is modified and the client is removed using
    Server::removeClient(SOCKET Socket)).

    In Server::processClientMessage(AbstractMessage *), as mentioned above,
    special processing can be performed in subclasses; only the ServerAcceptMessage
    is removed from memory here.

    Sending to the client:
    ---------------------
    Must be worked out in subclasses of Server or subclasses of ServerReceiverTransmitter
    or otherwise. At least all classes have access to the socket.

    Exit from Server:
    ----------------------
    First the server thread is stopped, then the ServerAcceptReceiver is removed
    from memory. Finally, all TcpServerReceivers are removed from the ClientMap and
    memory.
  \endif

  \if german
    Eine einfache TCP-Serverklasse, welche ihr KnowHow in den abgeleiteten Klassen
    lernen soll.

    Server ist ein Thread, welcher erlaubt, dass sich Client's mit ihm
    Verbinden.

    Server haben als Oberklassen-Attribute den Port, die Datengröße welche
    versendet wird, die Adressfamilie, den Sockettyp, den Protokoltyp und einen
    MainSocket (hier AcceptSocket).

    Zusätzlich sind hier als Attribute ein ServerAcceptReceiver, welcher die
    Neuanfragen als Thread bearbeitet, eine MsgQueue, welche die einzelnen Messages
    der einzelnen ClientSockets aufnimmt, ein Counter welcher die maximale Anzahl
    Clients beinhaltet und eine ClientMap mit den einzelnen
    ServerReceiverTransmitter-Objekten.

    Für jeden angenommenen Client wird ein Objekt der eingebetteten Klasse
    ServerReceiverTransmitter instanziiert, welche einen Transmitter und einen Receiver
    besitzt und in die ClientMap abgelegt. Transmitter und Receiver sind jeweils
    auch Threads.

    Start des Servers:
    ------------------
    Bei der instanziierung mit Übergabe von Port, Adressfamilie, Sockettyp, Protokol,
    Datengröße und maximale Clientanzahl wird der eigene Thread und der
    ServerAcceptReceiver initialisiert (mit dem aufrufenden Server als Elternobjekt
    und dem MainSocket ) und ebenfalls als Thread gestartet. Dieser blockiert immer
    solang, bis sich ein Client anmeldet.

    Neuer Client will sich verbinden:
    ---------------------------------
    Wenn sich ein Client beim ServerAcceptReceiver anmeldet, wird eine neue
    ServerAcceptMessage instanziiert (mit dem neuen SOCKET des Clients als
    Parameter) und an den Server über
    Server::acceptClientMessage(AbstractMessage *) weitergegeben.

    In Server::acceptClientMessage(AbstractMessage *) wird die übergebene
    ServerAcceptMessage in die MsgQueue angehängt. Beim abarbeiten der MsgQueue
    in Server::work() wird die ServerAcceptMessage gefunden.

    Wenn die maximale Clientanzahl überschritten ist, wird dieses dem Client
    mitgeteilt und der Clientsocket geschlossen.

    Wenn der Clientsocket nicht geschlossen, wird ein ServerReceiverTransmitter über eine
    virtuale Methode Server::generateNewServerReceiverTransmitter (...) instanziiert.
    Übergeben wird der Server als Elternobjekt für den noch zu instanziiernden
    ServerReceiver, der neue Socket und die Datengröße. Es werden für den
    ServerReceiverTransmitter ein Transmitter (der sendet Daten an den neuen Client) und ein
    ServerReceiver (der empfängt Daten des neuen Client) instanziiert. Es ist
    möglich einen abgeleiteten ServerReceiverTransmitter mit abgeleiteten Transmitter bzw.
    ServerReceiver zu etablieren.

    Der neue ServerReceiverTransmitter wird in die ClientMap eingefügt.

    Jetzt wird noch die Server-Methode
    Server::processClientMessage(AbstractMessage *) aufgerufen, in welcher in
    Unterklassen eine weitere Bearbeitung vorgenommen werden kann. Hier wird u.a.
    die ServerAcceptMessage aus dem Speicher entfernt.

    Für weitere Messages des Clients ist jetzt der neue ServerReceiverTransmitter bzw.
    dessen ServerReceiver in der ClientMap zuständig.

    Daten vom Client verarbeiten:
    -----------------------------
    Der ServerReceiver wartet in seiner Threadmethode ServerReceiver::work() auf
    Messages des verbundenen Clients. Wenn der Client sendet (Abbruch oder Daten),
    wird vom ServerReceiver eine neue ServerMessage über die Methode
    ServerReceiver::generateNewTcpMessage (...) instanziiert (kann in Unterklassen
    überschrieben werden, um abgeleitete MessageTypen zu liefern) und über Methode
    ServerReceiver::acceptMessage(AbstractMessage *) an den Parent-Server
    Server::acceptClientMessage(AbstractMessage *) weitergegeben.

    In Unterklassen kann ein abgeleitete ServerReceiver die Bearbeitung in der
    überschriebenen ServerReceiver::acceptMessage(AbstractMessage *) übernehmen
    und/oder der abgeleitete Server in der überschriebenen
    Server::acceptClientMessage(AbstractMessage *).

    In der Methode Server::acceptClientMessage(AbstractMessage *) wird die
    übergebene ServerMessage in die MsgQueue angehängt. Beim abarbeiten der
    MsgQueue wird die ServerMessage gefunden und an
    Server::processClientMessage(AbstractMessage *) weitergegeben. Wenn der
    Client die Verbindung unterbrochen hat, wird die Nachricht modifiziert und der
    Client über Server::removeClient(SOCKET Socket) entfernt).

    In Server::processClientMessage(AbstractMessage *) kann wie erwähnt in
    Unterklassen eine spezielle Bearbeitung vorgeneommen werden; hier wird nur die
    ServerAcceptMessage aus dem Speicher entfernt.

    Senden an den Client:
    ---------------------
    Muss in Unterklassen von Server bzw. Unterklassen von ServerReceiverTransmitter oder
    sonstwie ausgearbeitet werden. Zumindest haben alle Klassen Zugriff auf den
    Socket.

    Beenden vom Server:
    ----------------------
    Es wird zuerst der Server-Thread angehalten, dann der ServerAcceptReceiver aus
    dem Speicher entfernt. Zuletzt werden alle ServerReceiver aus der ClientMap
    und dem Speicher entfernt.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll Server : public TcpMain
  {
    public:
      //-----------------------------------------------------------------------
      // >>>>>>>>>>>>>>>>>> ServerReceiverTransmitter >>>>>>>>>>>>>>>>>>>>>>>>>
      //-----------------------------------------------------------------------
      /*! \brief
      \if english
        Tuple of socket Receiver and Transmitter for the Server.
      \endif

      \if german
        Tuple von Socket-Receiver und -Transmitter für den Server.
      \endif

      \copyright © 2010 Helmut Jakoby
      \author Helmut Jakoby
      */
      class __tcp_export_dll ServerReceiverTransmitter
      {
        private:
          //============== Attribute
          //---------------------------------------------------------------------
          /*!
          \if english
            The Tcp Receiver of the server (receives data from a Client).
          \endif

          \if german
            Der Tcp-Leser des Servers (empfängt Daten von einem Client)          
          \endif
          */
          ServerReceiver * m_pReceiver;
          //---------------------------------------------------------------------
          /*!
          \if english
            The Tcp Transmitter of the server (sends data to a Client).
          \endif

          \if german
            Der Tcp-Schreiber des Servers (sendet Daten an einen Client).
          \endif
          */
          Transmitter * m_pTransmitter;
          //---------------------------------------------------------------------

        protected:
          //============== Konstruktoren
          //---------------------------------------------------------------------
          /*!
          \if english
            This constructor is not available.
          \endif

          \if german
            Dieser Konstruktor steht niccht zur Verfügung.
          \endif
          */
          ServerReceiverTransmitter();
          //---------------------------------------------------------------------

        public:
          //============== Konstruktoren
          //---------------------------------------------------------------------
          /*!
          \if english
            This constructor starts the passed tcp Receiver and Transmitter.
            \param[in] pTransmitter The Transmitter (sends data to a Client).
            \param[in] pReceiver The Receiver (receives data from a Client).
            \attention ServerReceiverTransmitter assumes responsibility for the objects
              transferred in pTransmitter and pReceiver. This means that the calling
              instance must not remove the passed objects from memory!
            \exception eut::ErrorNException An exception is thrown if an error occurs.
          \endif

          \if german
            Dieser Konstruktor startet übergebenen Tcp-Leser und -Schreiber.
            \param[in] pTransmitter Der Schreiber (sendet Daten an einen Client).
            \param[in] pReceiver Der Leser (empfängt Daten von einem Client).
            \attention Der ServerReceiverTransmitter übernimmt die Verantwortung für die
              in pTransmitter und pReceiver übergebenen Objekte. Das bedeutet, die
              aufrufende Instanz darf die übergebenen Objekte nicht aus dem Speicher
              entfernen.
            \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
          \endif
          */
          ServerReceiverTransmitter( Transmitter * pTransmitter, ServerReceiver * pReceiver );
          //---------------------------------------------------------------------
          /*!
          \if english
            In the destructor the Tcp Receiver and Transmitter are stopped and removed
            from the memory.
          \endif

          \if german
            Im Destruktor werden der Tcp-Leser und -Schreiber angehalten und aus
            dem Speicher entfernt.
          \endif
          */
          virtual ~ServerReceiverTransmitter();
          //---------------------------------------------------------------------

        private:
          //============== Copy-Konstruktor
          //---------------------------------------------------------------------
          /*!
          \if english
            The copy constructor is not available.
          \endif

          \if german
            Der Copy-Konstruktor steht nicht zur Verfügung.
          \endif
          */
          ServerReceiverTransmitter( const ServerReceiverTransmitter & rT );
          //---------------------------------------------------------------------

        public:
          //============== Methoden
          //---------------------------------------------------------------------
          /*!
          \if english
            Sends transferred data to the client via its Transmitter.
            \param[in] pszRawMsg Pointer to the data to be sent.
            \param[in] uiRawMsgSize The size of the data to be sent.
            \param[in] uiMessageID A MessageID can be passed.
            \param[in] iActionCode A ActionCode can be passed.
            \param[in] rstrInfo A free info can be passed.
            \return A return value < 0 indicates an error.
            \attention The data in pszRawMsg is internally removed from memory; the
              calling instance may no longer use the data to be sent or remove it
              from memory.
          \endif

          \if german
            Sendet übergebene Daten über seinen Transmitter an den Client.
            \param[in] pszRawMsg Zeiger auf die zu sendenden Daten.
            \param[in] uiRawMsgSize Die Größe der zu sendenden Daten.
            \param[in] uiMessageID Es kann eine MessageID übergeben werden.
            \param[in] iActionCode Es kann ein ActionCode übergeben werden.
            \param[in] rstrInfo Es kann eine freie Info übergeben werden.
            \return Eine Rückgabe < 0 zeigt einen Fehler an.
            \attention Es werden die Daten in pszRawMsg intern aus dem Speicher
              entfernt; die aufrufende Instanz darf die zu sendenden Daten nicht
              mehr verwenden oder aus dem Speicher entfernen.
          \endif
          */
          int send( char * pszRawMsg,
                    unsigned int uiRawMsgSize,
                    unsigned int uiMessageID = 0,
                    int iActionCode = 0,
                    const std::string & rstrInfo = "" );
          //---------------------------------------------------------------------
          /*!
          \if english
            Sends passed Message to the client via its Transmitter.
            \param[in] pTcpMessage The message packet to be sent.
            \return A return < 0 indicates an error.
            \attention The passed Message must not be removed from memory by the
              calling instance! Since the Message is passed to the Transmitter for
              further processing, and the Transmitter decides when to send, the
              Transmitter must also remove the Message from memory.
          \endif

          \if german
            Sendet übergebene Message über seinen Transmitter an den Client.
            \param[in] pTcpMessage Das zu sendende Nachrichtenpacket.
            \return Eine Rückgabe < 0 zeigt einen Fehler an.
            \attention Die übergebene Message darf von der aufrufenden Instanz nicht
              aus dem Speicher entfernt werden! Da die Message zur Weiterverarbeitung
              an den Transmitter gegeben wird, und dieser entscheidet wann gesendet wird,
              muss dieser auch die Message aus dem Speicher entfernen.
          \endif
          */
          int send( Message * pTcpMessage );
          //---------------------------------------------------------------------
          /*!
          \if english
            Is called if socket is no longer ok.
          \endif

          \if german
            Wird aufgerufen, wenn Socket nicht mehr ok.
          \endif
          */
          void setValid( bool bValid );
          //---------------------------------------------------------------------
          /*!
          \if english
            Returns the IP address and the port.
            \param[in,out] rstrIPAdress The IP address (e.g. "LOCALHOST" or "192.168.2.102").
            \param[in,out] ruiPort The port which is addressed.
            \return A return value < 0 indicates an error.
          \endif

          \if german
            Liefert die IP-Adresse und den Port.
            \param[in,out] rstrIPAdress Die IP-Adresse (z.B. "LOCALHOST" oder "192.168.2.102").
            \param[in,out] ruiPort Der Port welcher angesprochen wird.
            \return Eine Rückgabe < 0 zeigt einen Fehler an.
          \endif
          */
          int getPeerName( std::string & rstrIPAdress, unsigned int & ruiPort ) const;
          //---------------------------------------------------------------------
          /*!
          \if english
            Returns the socket.
            \exception eut::ErrorNException If no socket, an exception is thrown.
          \endif

          \if german
            Liefert den Socket.
            \exception eut::ErrorNException Wenn kein SOCKET, wird eine Exception geworfen.
          \endif
          */
          SOCKET Socket() const;
          //---------------------------------------------------------------------
        private:
          //============== Operatoren
          //---------------------------------------------------------------------
          /*!
          \if english
            The assignment operator is not available.
          \endif

          \if german
            Der Zuweisungsoperator steht nicht zur Verfügung.
          \endif
          */
          Server::ServerReceiverTransmitter & operator= ( const ServerReceiverTransmitter & );
        //---------------------------------------------------------------------
      };
      //-----------------------------------------------------------------------
      // <<<<<<<<<<<<<<<<< ServerReceiverTransmitter <<<<<<<<<<<<<<<<<<<<<<<<<<
      //-----------------------------------------------------------------------

    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The maximum number of clients to accept. If the number is exceeded, no more
        clients are accepted.
      \endif

      \if german
        Die maximle Anzahl der anzunehmenden Clients. Wird die Anzahl überschritten,
        werden keine Clients mehr angenommen.
      \endif
      */
      unsigned long m_ulMaxClients;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The queue with the received data from the client.
      \endif

      \if german
        Die Queue mit den empfangenen Daten vom Client.
      \endif
      */
      std::queue<AbstractMessage *> m_MsgQueue;
      //-----------------------------------------------------------------------

    protected:
      //-----------------------------------------------------------------------
      /*!
      \if english
        The AcceptReceiver accepts new clients.
      \endif

      \if german
        Der AcceptReceiver nimmt neue Clients an.
      \endif
      */
      ServerAcceptReceiver * m_pAcceptReceiver;
      //-----------------------------------------------------------------------
      /*!
      \if english
        All accepted clients in Server::ServerReceiverTransmitter.
      \endif

      \if german
        Alle angenommenen Clients im Server::ServerReceiverTransmitter.
      \endif
      */
      std::map< SOCKET, ServerReceiverTransmitter *, std::less< SOCKET > > m_ClientMap;
      //-----------------------------------------------------------------------

      //============== Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Server();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Server( unsigned int, int, int, int, int );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Server( unsigned int, int, int, int, unsigned int, int );
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor starts the server.
        \param[in] uiPort The port of the server.
        \param[in] iAdressFamily The socket address family to instantiate a socket
          (usually AF_INET).
        \param[in] iSocketType The socket type (usually SOCK_STREAM).
        \param[in] iProtocol The socket protocol (usually 0).
        \param[in] uiDataSize The packet size.
        \param[in] ulMaxClients The maximum number of clients allowed to connect.
        \param[in] iSecTimeOut The time until waiting for a connection or waiting
          for data is aborted.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Dieser Konstruktor startet den Server.
        \param[in] uiPort Der Port des Servers.
        \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
          instanziieren (i.d.R. AF_INET).
        \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM).
        \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
        \param[in] uiDataSize Die Paketgröße.
        \param[in] ulMaxClients Die maximale Anzahl von Clients, die sich verbinden
          dürfen.
        \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
           warten auf Daten abgebrochen wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      Server( unsigned int uiPort,
              int iAdressFamily,
              int iSocketType,
              int iProtocol,
              unsigned int uiDataSize,
              unsigned long ulMaxClients,
              int iSecTimeOut );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor. It is cleaned up (AcceptReceiver and MsgQueue).
      \endif

      \if german
        Destruktor. Es wird aufgeräumt (AcceptReceiver und MsgQueue).
      \endif
      */
      virtual ~Server();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Server( const Server & rT );
      //-----------------------------------------------------------------------

      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Initializes the socket and instantiates a ServerAcceptReceiver. The own
        attributes must have meaningful values.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Initialisiert den Socket und instanziiert einen ServerAcceptReceiver. Die
        eigenen Attribute müssen sinnvolle Werte haben.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      void init();
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Work of the thread. New clients will be 'accepted' or rejected when the
        maximum number is reached. Messages from clients are processed.
      \endif

      \if german
        Arbeit des Threads. Es werden neue Clients 'aufgenommen' bzw. abgelehnt
        wenn die max. Anzahl erreicht. Es werden Nachrichten von Clients abgearbeitet.
      \endif
      */
      virtual void work() override;
      //-----------------------------------------------------------------------
      #if defined (__PTHREADS)
      /*!
      \if english
        When the thread ends, this virtual method is called.
        The MsgQueue is cleaned up here.
        \note This method only exists when compiled with POSIX threads (defined
          (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        Hier wird die MsgQueue aufgeräumt.
        \note Diese Methode gibt es nur, wenn mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
      virtual void cleanUp();
      //-----------------------------------------------------------------------
      #else

    public:
      //============== Methoden
    //-------------------------------------------------------------------------
      /*!
      \if english
        When the thread ends, this virtual method is called.
        The MsgQueue is cleaned up here.
        \note This method only exists when not compiled with POSIX threads (defined
          (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        Hier wird die MsgQueue aufgeräumt.
        \note Diese Methode gibt es nur, wenn nicht mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
      virtual void cancel() override;
      //-----------------------------------------------------------------------
      #endif

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Adds new client as 'ServerReceiverTransmitter' to its list of clients. This method
        is virtual, so that subclasses can react to it; e.g. let someone know that
        a new client has arrived.
        \param[in] pNewServerReceiverTransmitter The client is in this object.
      \endif

      \if german
        Fügt neuen Client als 'ServerReceiverTransmitter' in seine Liste der Clients ein.
        Diese Methode ist virtual, damit in Unterklasse darauf reagieren werden
        kann; z.B. irgendwem mitteilen, dass ein neuer Client da ist.
        \param[in] pNewServerReceiverTransmitter Der Client ist in diesem Objekt.
      \endif
      */
      virtual void insertNewClient( Server::ServerReceiverTransmitter * pNewServerReceiverTransmitter );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Looks for the ServerReceiverTransmitter corresponding to the 'Socket' parameter
        in its list of clients and returns a pointer to it, if any.
        \param[in] Socket The socket of the ServerReceiverTransmitter being searched for.
        \return If != 0, then the ServerReceiverTransmitter found.
        \attention The returned object only lives as long as the returning object
          and must NOT be removed from memory by the calling instance.
      \endif

      \if german
        Sucht den mit Parameter 'Socket' korrespondierenden ServerReceiverTransmitter in
        seine Liste der Clients und liefert einen Zeiger auf diesen, wenn
        vorhanden.
        \param[in] Socket Der Socket des gesuchten ServerReceiverTransmitter.
        \return Wenn != 0, dann der gefundene ServerReceiverTransmitter.
        \attention Das gelieferte Objekt lebt nur so lange wie das liefernde Objekt
          und darf NICHT von der aufrufenden Instanz aus dem Speicher entfernt werden.
      \endif
     */
      Server::ServerReceiverTransmitter * getClient( SOCKET Socket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Looks for the ServerReceiverTransmitter corresponding to the 'Socket' parameter
        in its list of clients and removes it from the list and from memory.
        \param[in] Socket The socket to be closed.
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Sucht den mit Parameter 'Socket' korrespondierenden ServerReceiverTransmitter in
        seine Liste der Clients und entfernt diesen daraus und aus dem Speicher.
        \param[in] Socket Der Socket, welcher geschlossen werden soll.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
     */
      int virtual removeClient( SOCKET Socket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a ServerReceiverTransmitter here; can be overwritten in subclasses to
        return a derived type. Is called in work() when a new client is added.
        \param[in] pParent Server or subclass.
        \param[in] Socket Socket of the new client.
        \param[in] iDataSize The packet size.
        \return The ServerReceiverTransmitter or derived type required in the subclasses.
      \endif

      \if german
        Liefert hier einen ServerReceiverTransmitter; kann in Unterklassen überschrieben
        werden um einen abgeleiteten Typ liefern. Wird in work() aufgerufen, wenn
        ein neuer Client hinzukommt.
        \param[in] pParent Server oder Unterklasse.
        \param[in] Socket Socket des neuen Clients.
        \param[in] iDataSize Die Packetgröße.
        \return Der ServerReceiverTransmitter bzw. in den Unterklassen benötigter abgeleiteter
          Typ.
      \endif
      */
      virtual Server::ServerReceiverTransmitter * generateNewServerReceiverTransmitter( Server * pParent,
                                                                                        SOCKET Socket,
                                                                                        unsigned int iDataSize ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the Valid flag for all ServerReceiverTransmitters from its list of clients.
        \param[in] bValid If false -> no more reception!
      \endif

      \if german
        Setzt bei allen ServerReceiverTransmitter aus seiner Liste der Clients das
        Valid-Flag.
        \param[in] bValid Wenn false -> kein Empfang mehr!
      \endif
      */
      void setClientMapReceiverValid( bool bValid );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes all ServerReceiverTransmitters from its list of clients and from memory.
      \endif

      \if german
        Entfernt alle ServerReceiverTransmitter aus seiner Liste der Clients und aus dem
        Speicher.
      \endif
      */
      void clearClientMap();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the maximum number of client connections allowed.
        \return The maximum number of client connections allowed.
      \endif

      \if german
        Liefert die Anzahl der maximal zulässigen Clientverbindungen.
        \return Die maximal zulässige Anzahl der Clientverbindungen.
      \endif
      */
      unsigned long getMaxClients() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Should process the message from the client in subclasses. In the overwritten
        methods <B>always</B> this method must be called, because among other things
        the passed Message is removed from the memory here.
        \param[in] pTcpMessage The message to be processed by the client.
        \return A return < 0 indicates an error.
      \endif

      \if german
        Soll in Unterklassen die Nachricht vom Client verarbeiten. In den
        überschriebenen Methoden mus <B>immer</B> diese Methode aufgerufen werden,
        weil hier u.a. die übergebene Message aus dem Speicher entfernt wird.
        \param[in] pTcpMessage Die zu verarbeitende Nachricht vom Client.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      virtual void processClientMessage (AbstractMessage * pTcpMessage);
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sends the transferred data in a Message via its Transmitter to the client.
        A header will be generated.
        \param[in] ClientSocket The socket of the target client.
        \param[in] pszRawMsg Pointer to the data to be sent.
        \param[in] uiRawMsgSize The size of the data to be sent.
        \param[in] uiMessageID A MessageID can be passed.
        \param[in] iActionCode A ActionCode can be passed.
        \param[in] rstrInfo A free info can be passed.
        \return A return value < 0 indicates an error.
        \attention The data in pszRawMsg is internally removed from memory; the
          calling instance may no longer use the data to be sent or remove it from
          memory.
      \endif

      \if german
        Sendet übergebene Daten über seinen Transmitter an den Client. Es
        wird ein Header generiert.
        \param[in] ClientSocket Socket des Zielclients.
        \param[in] pszRawMsg Zeiger auf die zu sendenden Daten.
        \param[in] uiRawMsgSize Die Größe der zu sendenden Daten.
        \param[in] uiMessageID Es kann eine MessageID übergeben werden.
        \param[in] iActionCode Es kann ein ActionCode übergeben werden.
        \param[in] rstrInfo Es kann eine freie Info übergeben werden.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Es werden die Daten in pszRawMsg intern aus dem Speicher entfernt;
          die aufrufende Instanz darf die zu sendenden Daten nicht mehr verwenden oder
          aus dem Speicher entfernen.
      \endif
      */
      virtual int send( SOCKET ClientSocket,
                        char * pszRawMsg,
                        unsigned int uiRawMsgSize,
                        unsigned int uiMessageID = 0,
                        int iActionCode = 0,
                        const std::string & rstrInfo = "" );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sends the transferred Message to the client via its Transmitter (Client
        information is in the ServerMessage).
        \param[in] pTcpMessage The message packet to be sent must be a ServerMessage.
        \return A return < 0 indicates an error.
        \attention The passed Message must not be removed from memory by the
          calling instance! Since the Message is given to the Transmitter for further
          processing, and the Transmitter decides when to send. Therefore, the Transmitter
          will also remove the Message from memory.
      \endif

      \if german
        Sendet übergebene Message über seinen Transmitter an den Client (ersichlich
         aus der ServerMessage).
        \param[in] pTcpMessage Das zu sendende Nachrichtenpacket, muss eine
          ServerMessage sein.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Die übergebene Message darf von der aufrufenden Instanz nicht
          aus dem Speicher entfernt werden! Da die Message zur Weiterverarbeitung
          an den Transmitter gegeben wird, und dieser entscheidet wann gesendet wird.
          Darum wird dieser auch die Message aus dem Speicher entfernen.
      \endif
      */
      virtual int send( Message * pTcpMessage ) override;
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        The transferred Message is inserted into the MsgQueue by the client via
        the ServerAcceptReceiver or ServerReceiver and is thus processed.
        \param[in] pTcpMessage The received message from the client.
        \return A return < 0 indicates an error.
      \endif

      \if german
        Es wird die übergebene Message vom Client über den ServerAcceptReceiver
        oder ServerReceiver in die MsgQueue eingefügt und somit verarbeitet.
        \param[in] pTcpMessage Die erhaltene Nachricht vom Client.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int acceptClientMessage( AbstractMessage * pTcpMessage );
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Server & operator= ( const Server & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( default : 4275 )
  #endif
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
