#ifndef INC_TCPSERVERRECEIVER_H
#define INC_TCPSERVERRECEIVER_H
//-----------------------------------------------------------------------------
/*!
\file TcpServerReceiver.h
\brief
\if english
  Header for ServerReceiver.
\endif

\if german
  Header für ServerReceiver.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpDataReceiver.h"

#include "TcpMain.h"

#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  //Forwards
  class Server;
  class AbstractMessage;
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Class that communicates with a Client via a SOCKET and delivers its receive
    result of type ServerMessage to its "owner", which must be a Server. You
    can use the method ServerReceiver::acceptMessage(AbstractMessage * pTcpMessage)
    to change the delivery to the "owner" in subclasses by overwriting the method.
  \endif

  \if german
    Klasse, welche über ein SOCKET mit einem Client kommunizieret und ihr
    Empfangsergebnis vom Typ ServerMessage ist, an ihren "Besitzer", welcher ein
    Server sein muss, liefert. Das Liefern an den "Besitzer" kann über Methode
    ServerReceiver::acceptMessage(AbstractMessage * pTcpMessage) in Unterklassen
    durch Überschreiben der Methode abgeändert werden.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll ServerReceiver : public DataReceiver
  {
    protected:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        Set to false if socket is no longer ok.
      \endif

      \if german
        Wird ggf. auf false gesetzt, wenn Socket nicht mehr ok.
      \endif
      */
      bool m_bValid;
      //-----------------------------------------------------------------------

      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerReceiver();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerReceiver( SOCKET );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerReceiver( TcpMain *, SOCKET );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerReceiver( TcpMain *, SOCKET, unsigned int );
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] pParent The owning parent object.
        \param[in] Socket The socket through which communication takes place.
        \param[in] uiDataSize The package size.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] pParent Das besitzende Elternobjekt.
        \param[in] Socket Der Socket über den kommuniziert wird.
        \param[in] uiDataSize Die Paketgröße.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      ServerReceiver( Server * pParent, SOCKET Socket, unsigned int uiDataSize );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~ServerReceiver();
      //-----------------------------------------------------------------------


    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerReceiver( const ServerReceiver & );
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Accepts streams from its socket (is blocked until received), turns the
        data into a ServerMessage which is further processed in 
        ServerReceiver::acceptMessage(AbstractMessage * pTcpMessage).
      \endif

      \if german
        Nimmt von seinem Socket Streams entgegen (ist bis zu Empfang blockiert),
        macht aus den Daten eine ServerMessage welche in
        ServerReceiver::acceptMessage(AbstractMessage * pTcpMessage) weiter
        verarbeitet wird.
      \endif
      */
      virtual void work() override;
      //-----------------------------------------------------------------------
      #if defined (__PTHREADS)
      /*
      \if english
        When the thread ends, this virtual method is called.
        \note This method only exists when compiled with POSIX threads (defined (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        \note Diese Methode gibt es nur, wenn mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
        virtual void cleanUp();
      #endif
      //-----------------------------------------------------------------------
      /*!
      \if english
        Forwards the transferred message to its Server, which also deletes the
        message after processing. <br>
        The Server can then process these further in subclasses if necessary.
        If processing in a subclass of ClientReceiver is required, this method
        can also be overwritten.
        \param[in] pTcpServerMessage The received message.
        \return A return < 0 indicates an error.
        \attention The passed object must not be removed from memory by the calling
          instance.
      \endif

      \if german
        Leitet die übergebene Nachricht an seinen Server weiter, der die Nachricht
        nach der Verarbeitung auch löscht. <br>
        Der Server kann diese dann ggf. in Unterklassen weiter verarbeiten. Wenn
        die Bearbeitung in einer Unterklasse von ClientReceiver gewünscht wird,
        kann diese Methode auch überschrieben werden.
        \param[in] pTcpServerMessage Die empfangene Nachricht.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Das übergebene Objekt darf von der aufrufenden Instanz nicht aus dem
          Speicher entfernt werden.
      \endif
      \sa ServerMessage
      */
      virtual int acceptMessage( AbstractMessage  * pTcpServerMessage ) override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Generates a ServerMessage from the transferred data buffer and its size.
        \param[in,out] prTcpServerMessage Pointer to newly generated ServerMessage.
          This object takes care of removing the pszTransmitData from the memory.
        \param[in] pszTransmitData The data buffer including all required
          information.
        \param[in] uiTransmitDataSize The size or length of the data buffer to be
          sent.
        \attention The supplied object must be removed from memory by the calling
          instance.
        \exception eut::ErrorNException If no ServerMessage can be instantiated from the data
          (because, for example, the header is incorrect), an exception is thrown.
      \endif

      \if german
        Generiert eine ServerMessage aus Datenpuffer und dessen Größe.
        \param[in,out] prTcpServerMessage Zeiger auf neu generierte ServerMessage.
          Dieses Objekt übernimmt das entfernen der pszTransmitData aus dem Speicher.
        \param[in] pszTransmitData Der Datenpuffer inkl. aller benötigten
          Informationen.
        \param[in] uiTransmitDataSize Die Größe bzw. Länge des zu sendenden Datenpuffers.
        \attention Das gelieferte Objekt muss von der aufrufenden Instanz aus dem
          Speicher entfernt werden.
        \exception eut::ErrorNException Wenn aus den Daten keine ServerMessage instantiiert
          werden kann (weil z.B. der Header nicht stimmt), wird eine Exception
          geworfen.
      \endif
      \sa ServerMessage
      */
      virtual void generateNewTcpMessage( AbstractMessage *& prTcpServerMessage,
                                          char * pszTransmitData,
                                          unsigned int uiTransmitDataSize ) const override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Generates a ServerMessage from transferred parameters.
        \param[in,out] prTcpServerMessage Pointer to newly generated ServerMessage.
          This object takes over the removal of the pszRawMsg from the memory.
        \param[in] pszRawMsg The message data.
        \param[in] uiRawMsgSize The size or length of the message data.
        \param[in] uiMessageID A unique ID.
        \param[in] iActionCode The ActionCode.
        \param[in] rstrInfo An additional text info
        \attention The supplied object must be removed from memory by the calling
          instance.
        \exception eut::ErrorNException If no ServerMessage can be instantiated from the
          parameters, an exception is thrown.
      \endif

      \if german
        Generiert eine ServerMessage aus übergebenen Parametern.
        \param[in,out] prTcpServerMessage Zeiger auf neu generierte ServerMessage.
          Dieses Objekt übernimmt das entfernen der pszRawMsg aus dem Speicher.
        \param[in] pszRawMsg Die Nachrichtendaten.
        \param[in] uiRawMsgSize Die Größe bzw. Länge der Nachrichtendaten.
        \param[in] uiMessageID Eine eindeutige ID.
        \param[in] iActionCode Der ActionCode.
        \param[in] rstrInfo Eine zusätzliche Text-Info
        \attention Das gelieferte Objekt muss von der aufrufenden Instanz aus dem
          Speicher entfernt werden.
        \exception eut::ErrorNException Wenn aus den Parametern keine ServerMessage instantiiert
          werden kann, wird eine Exception geworfen.
      \endif
      \sa ServerMessage
      */
      virtual void generateNewTcpMessage( AbstractMessage *& prTcpServerMessage,
                                          char * pszRawMsg,
                                          unsigned int uiRawMsgSize,
                                          unsigned int uiMessageID,
                                          int iActionCode,
                                          const std::string & rstrInfo ) const override;
      //-----------------------------------------------------------------------

    public:
      //========== Methoden ========
      //-----------------------------------------------------------------------
      /*!
      \if english
        Terminates the thread.
      \endif

      \if german
        Beendet den Thread.
      \endif
      */
      #if defined (__PTHREADS)
        virtual int cancel();
      #else
        virtual void cancel() override;
      #endif
      //-----------------------------------------------------------------------
      /*!
      \if english
        Called when socket is no longer ok.
        \param[in] bValid If there is a SOCKET error, false is passed here.
      \endif

      \if german
        Wird aufgerufen, wenn Socket nicht mehr ok.
        \param[in] bValid Wenn es einen SOCKET Fehler gibt, wird hier false übergeben.
      \endif
      */
      void setValid( bool bValid );
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      ServerReceiver & operator= ( const ServerReceiver & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
