#ifndef INC_TCPTRANSMITTER_H
#define INC_TCPTRANSMITTER_H
//-----------------------------------------------------------------------------
/*!
\file TcpTransmitter.h
\brief
\if english
  Header for Transmitter.
\endif

\if german
  Header für Transmitter.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpReceiverTransmitter.h"
#include "TcpMessage.h"
#include <queue>
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( disable : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( disable : 4275 )
  #endif
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    A class that communicates with a Client or Server via a SOCKET and
    expects send data of type Message.
  \endif

  \if german
    Eine Klasse, welche über ein SOCKET mit einem Client oder Server
    kommunizieret und Senddaten vom Typ Message erwartet.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll Transmitter : public ReceiverTransmitter
  {
    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The queue with the messages to be sent.
      \endif

      \if german
        Die Queue mit den zu sendenden Nachrichten.
      \endif
      */
      std::queue<Message *> m_MsgQueue;
      //-----------------------------------------------------------------------

    protected:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The standard constructor is not available.
      \endif

      \if german
        Der Standard-Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Transmitter();
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] Socket The socket via which communication takes place.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] Socket Socket über den kommuniziert wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      Transmitter( SOCKET Socket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~Transmitter();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Transmitter( const Transmitter & );
      //-----------------------------------------------------------------------

    protected:
      //========== Methoden ========
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sends the data from the send message list to the socket passed in the
        constructor.
      \endif

      \if german
        Sendet die Daten aus der Liste der Sendenachrichten an das Socket, der im
        Konstruktor übergeben wurde.
      \endif
      */
      virtual void work() override;
      //-----------------------------------------------------------------------
      #if defined (__PTHREADS)
      /*!
      \if english
        When the thread ends, this virtual method is called.
        Here the list of send messages is cleaned up.
        \note This method only exists when compiled with POSIX threads (defined
          (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        Hier wird die Liste der Sendenachrichten aufgeräumt.
        \note Diese Methode gibt es nur, wenn mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
      virtual void cleanUp();
      //-----------------------------------------------------------------------
      #else

    public:
      //============== Methoden
      //------------------------------------------------------------------------
      /*!
      \if english
        When the thread ends, this virtual method is called.
        Here the list of send messages is cleaned up.
        \note This method only exists when not compiled with POSIX threads (defined
          (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        Hier wird die Liste der Sendenachrichten aufgeräumt.
        \note Diese Methode gibt es nur, wenn nicht mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
      virtual void cancel() override;
      //-----------------------------------------------------------------------
      #endif

    public:
      //============== Methoden
      //------------------------------------------------------------------------
      /*!
      \if english
        Includes the message in the list of send messages.
        \param[in] pTcpMessage The message to be sent.
        \return A return < 0 indicates an error or information.
        \attention The passed object is taken over by this object and must not be
          changed or removed from memory by the calling instance.
      \endif

      \if german
        Übernimmt die Nachricht in die Liste der Sendenachrichten.
        \param[in] pTcpMessage Die zu versendende Nachricht.
        \return Eine Rückgabe < 0 zeigt einen Fehler bzw. ein Info an.
        \attention Das übergebene Objekt wird von diesem Objekt übernommen und darf
          von der aufrufenden Instanz nicht mehr verändert oder aus dem Speicher
          entfernt werden.
      \endif
      */
      int acceptOrder( Message * pTcpMessage );
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Transmitter & operator= ( const Transmitter & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( default : 4275 )
  #endif
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
