//-----------------------------------------------------------------------------
/*!
\file TcpClient.cpp
\brief Source for Client
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
// @@End
#include "TcpClient.h"
#include "TcpClientReceiver.h"
#include "TcpTransmitter.h"
#include "EuException.h"
#include <iostream>
#include <sstream>
#include <cstring>
#include <ctime>
#if defined(_MSC_VER) || defined(__MINGW32__)
  #include <windows.h>
#elif defined(__GNUC__)
  #include <unistd.h>
  #define Sleep(x) usleep( (x)* 1000 )
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER TcpMain
  //---------------------------------------------------------------------------
  Client::Client ()
  : SUPER(),
    m_pReceiver( NULL_PTR ),
    m_pTransmitter( NULL_PTR ),
    m_eToWaitingForServer(ACTION_UNDEF)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::Client";
    #endif
  }
  //---------------------------------------------------------------------------
  Client::Client( unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  int iSecTimeOut )
  : SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, iSecTimeOut ),
    m_pReceiver( NULL_PTR ),
    m_pTransmitter( NULL_PTR ),
    m_eToWaitingForServer( ACTION_UNDEF )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::Client";
    #endif
  }
  //---------------------------------------------------------------------------
  Client::Client( const std::string & rsHostname,
                  unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  int iSecTimeOut )
  : SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, iSecTimeOut ),
    m_pReceiver( NULL_PTR ),
    m_pTransmitter( NULL_PTR ),
    m_sHostname( rsHostname ),
    m_eToWaitingForServer( ACTION_UNDEF )
  {
    int t_iErr = 0;

    if ( rsHostname.empty() )
    {
      throw eut::ErrorNException( "tcp::Client ctor()", ERR_NO_HOSTNAME, __FILE__, __LINE__ );
    }
    t_iErr = this->getSocketAdrFromString( & m_Socketadress, m_sHostname.c_str() );
    if ( t_iErr )
    {
      this->setWSACleanup();
      throw eut::ErrorNException( "tcp::Client ctor()", t_iErr, __FILE__, __LINE__ );
    }
    // eigenen Thread starten
    this->start();
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::Client";
    #endif
  }
  //---------------------------------------------------------------------------
  Client::Client( const std::string & rsHostname,
                  unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  unsigned int uiDataSize,
                  int iSecTimeOut )
  : SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, uiDataSize, iSecTimeOut ),
    m_pReceiver( NULL_PTR ),
    m_pTransmitter( NULL_PTR ),
    m_sHostname( rsHostname ),
    m_eToWaitingForServer(ACTION_UNDEF)
  {
    int t_iErr = 0;

    if ( rsHostname.empty() )
    {
      throw eut::ErrorNException( "tcp::Client ctor()", ERR_NO_HOSTNAME, __FILE__, __LINE__ );
    }
    t_iErr = this->getSocketAdrFromString( & m_Socketadress, m_sHostname.c_str() );
    if ( t_iErr )
    {
      this->setWSACleanup();
      throw eut::ErrorNException( "tcp::Client ctor()", t_iErr, __FILE__, __LINE__ );
    }
    // eigenen Thread starten
    this->start();
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::Client";
    #endif
  }
  //---------------------------------------------------------------------------
  Client::~Client ()
  {
    {
      this->disconnectServer();
    }
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class TcpMain' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  Client::Client( const Client & )
  : m_pReceiver( NULL_PTR ),
    m_pTransmitter( NULL_PTR ),
    m_eToWaitingForServer(ACTION_UNDEF)
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class TcpMain' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  void Client::work()
  {
    Message  * t_pTcpMessage( NULL_PTR );

    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
    #endif
    this->waitForInput();
    #if defined (__PTHREADS)
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    //^^^^^^^^^^^^^^^^^^^^
    #if defined (__PTHREADS)
      if ( m_eThreadOrder != EuPTHREAD_TAKE_A_BREAK )
    #else
      if ( this->getStatus() == eWORKING )
    #endif
    {
      //++++++++++++++++++++++++++++
      this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( ! m_MsgQueue.empty() )
      {
        t_pTcpMessage = m_MsgQueue.front();
        m_MsgQueue.pop();
        if ( m_eToWaitingForServer )
        {
          if  ( t_pTcpMessage->getActionCode() != m_eToWaitingForServer )
          {
            if  ( t_pTcpMessage->getActionCode() != ERR_TO_MUCH_CLIENTS
                  &&
                  t_pTcpMessage->getActionCode() != ERR_WSAECONNABORTED /*ERR_CONNECTION_BROKEN*/ )
            {
              m_MsgQueue.push( t_pTcpMessage );
              t_pTcpMessage = NULL_PTR;
            }
            this->signalInput();
          }
        }
      }
      //++++++++++++++++++++++++++++
      this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++

      if ( t_pTcpMessage )
      {
        //cout << "TcpMsg Laenge:" << t_pTcpMessage->m_lLength << endl;
        if ( t_pTcpMessage->getActionCode() < 0 )
        {
          // wenn gar nicht angenommen wegen zu vielen Clients, dann nicht this->disconnect();
          // das Aufräumen geschieht schon im aufrufenden ::connect()
          m_eToWaitingForServer = ACTION_UNDEF;
          if ( t_pTcpMessage->getActionCode() != ERR_TO_MUCH_CLIENTS )
          {
            this->disconnectServer();
          }
        }
        if  ( t_pTcpMessage->getActionCode() == m_eToWaitingForServer
              ||
              t_pTcpMessage->getActionCode() == ERR_TO_MUCH_CLIENTS )
        {
          // das erwartete ist da
          m_eToWaitingForServer = ACTION_UNDEF;
        }
        this->processServerMessage( t_pTcpMessage );
      }
    }
  }
  //----------------------------------------------------------------------
  #if defined (__PTHREADS)
  void Client::cleanUp()
  {
    SUPER::cleanUp();
    while ( ! m_MsgQueue.empty() )
    {
      delete m_MsgQueue.front();
      m_MsgQueue.pop();
    }
  }
  //---------------------------------------------------------------------------
  #else
  //---------------------------------------------------------------------------
  void Client::cancel()
  {
    while ( ! m_MsgQueue.empty() )
    {
      delete m_MsgQueue.front();
      m_MsgQueue.pop();
    }
    SUPER::cancel();
  }
  #endif
  //---------------------------------------------------------------------------
  Transmitter * Client::generateNewTransmitter()
  {
    Transmitter * t_pNewTransmitter = new Transmitter( m_MainSocket );

    return t_pNewTransmitter;
  }
  //---------------------------------------------------------------------------
  ClientReceiver * Client::generateNewReceiver()
  {
    ClientReceiver * t_pNewReceiver = new ClientReceiver( this, m_MainSocket, m_uiDataSize );

    return t_pNewReceiver;
  }
  //---------------------------------------------------------------------------
  void Client::setDataSize( unsigned int uiDataSize )
  {
    if ( m_pReceiver )
    {
      m_pReceiver->setDataSize( uiDataSize );
    }
    SUPER::setDataSize( uiDataSize );
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Konvertierung von "xxx" in "yyy", möglicher Datenverlust
    // m_Socketadress.sin_family = m_iAdressFamily;
    #pragma warning( disable : 4244 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsequence-point"
  #endif
  //---------------------------------------------------------------------------
  int Client::connectServer()
  {
    int t_iErr = 0;

    // wenn noch nicht gestartet...
    if ( ! m_MainSocket )
    {
      m_MainSocket = socket( m_iAdressFamily, m_iSocketType, m_iProtocol );
      if ( m_MainSocket == INVALID_SOCKET )
      {
        #if defined(_MSC_VER) || defined(__MINGW32__)
          t_iErr = WSAGetLastError();
        #elif defined(__GNUC__)
          t_iErr = errno;
        #endif
        m_MainSocket = 0;
      }
      else
      {
        memset( & m_Socketadress, 0, sizeof( SOCKADDR_IN ) ); // zuerst alles auf 0 setzten
        m_Socketadress.sin_family = m_iAdressFamily;
        m_Socketadress.sin_port = htons( m_uiServerPort );
        t_iErr = this->getSocketAdrFromString( & m_Socketadress, m_sHostname.c_str() );
        if ( ! t_iErr )
        {
          // Verbindung aufbauen...
          t_iErr = connect( m_MainSocket, ( SOCKADDR * ) & m_Socketadress, sizeof( SOCKADDR ) );
          if ( t_iErr == SOCKET_ERROR )
          {
            m_MainSocket = 0;
            #if defined(_MSC_VER) || defined(__MINGW32__)
              t_iErr = WSAGetLastError();
            #elif defined(__GNUC__)
              t_iErr = errno;
            #endif
            // wenn positiver Wert aus WSAGetLastError() hier auf Minus
            t_iErr = ( t_iErr > 0 ? t_iErr = t_iErr * -1 : t_iErr );
          }
          if ( ! t_iErr )
          {
            m_pTransmitter = this->generateNewTransmitter();
            m_pReceiver = this->generateNewReceiver();
            if ( m_pTransmitter )
            {
              t_iErr = m_pTransmitter->start();
            }
            if ( ! t_iErr && m_pReceiver )
            {
              t_iErr = m_pReceiver->start();
            }
          }
        }
      }
      if ( ! t_iErr )
      {
        if ( m_pReceiver->connected() )
        {
          t_iErr = this->syncDataSize();
        }
      }
      else
      {
        t_iErr = this->disconnectServer();
      }
    }
    else
    {
      t_iErr = ERR_SOCKET_ALREADY_EXISTING;
    }
    // letzte Prüfung; wenn m_pReceiver keinen Socket, dann ist nach dem connect die Verbindung
    // vom Server abgebrochen worden.
    if ( m_pReceiver && ! m_pReceiver->connected() )
    {
      t_iErr = ERR_TO_MUCH_CLIENTS;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Konvertierung von "xxx" in "yyy", möglicher Datenverlust
    #pragma warning( default : 4244 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  int Client::disconnectServer ()
  {
    int t_iErr = 0;

    //++++++++++++++++++++++++++++
    this->setExclusiveAccess();
    //++++++++++++++++++++++++++++
    if ( m_MainSocket )
    {
      if ( m_pReceiver )
      {
        m_pReceiver->cancel();
        delete m_pReceiver;
        m_pReceiver = NULL_PTR;
      }
      if ( m_pTransmitter )
      {
        m_pTransmitter->cancel();
        delete m_pTransmitter;
        m_pTransmitter = NULL_PTR;
      }
      m_MainSocket = 0;
      //m_iAdressFamily = 0;
      //m_iSocketType = 0;
      //m_iProtocol = 0;
    }
    else
    {
      t_iErr = ERR_SOCKET_NOT_EXISTING;
    }
    //++++++++++++++++++++++++++++
    this->unsetExclusiveAccess();
    //++++++++++++++++++++++++++++
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  bool Client::isServerConnected () const
  {
    bool  t_bRetVal(false);

    if ( m_MainSocket ) t_bRetVal = true;
    return t_bRetVal;
  }
  //---------------------------------------------------------------------------
  int Client::send( char * pszRawMsg,
                    unsigned int uiRawMsgSize,
                    unsigned int uiMessageID,
                    int iActionCode,
                    const std::string & rsInfo )
  {
    int   t_iErr = 0;

    #if defined (__PTHREADS)
      if ( this->status() != EuPTHREAD_WORKING )
      {
        t_iErr = EuPTHREAD_ERR_NO_THREAD;
      }
    #else
      if ( this->status() != eWORKING )
      {
        t_iErr = EuTHREAD_ERR_NO_THREAD;
      }
    #endif
    else if ( ! m_pTransmitter )
    {
      t_iErr = ERR_NO_TRANSMITTER;
    }
    else
    {
      Message * t_pMessage = new Message( pszRawMsg, uiRawMsgSize, uiMessageID, iActionCode, rsInfo );
      #ifdef _DEBUG_TcpMessage
        t_pMessage->m_sDebugInstanceInfo = "tcp::Client::send (char * pszRawMsg...)";
      #endif
        t_iErr = this->send( t_pMessage );
      #ifdef _DEBUG_THREAD
        std::cout << "tcp::Client::send(...) - Auftrag uebernommen" << std::endl;
      #endif
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Client::send( Message * pTcpMessage )
  {
    int   t_iErr = 0;

    if ( ! pTcpMessage )
    {
      t_iErr = ERR_PARAMETER_NULL_FORBIDDEN;
    }
    #if defined (__PTHREADS)
      else if ( this->status() != EuPTHREAD_WORKING )
      {
        t_iErr = EuPTHREAD_ERR_NO_THREAD;
      }
    #else
      else if ( this->status() != eWORKING )
      {
        t_iErr = EuTHREAD_ERR_NO_THREAD;
      }
    #endif
    else if ( ! m_pTransmitter )
    {
      t_iErr = ERR_NO_TRANSMITTER;
    }
    else
    {
      // t_pMessage wird im m_pTransmitter wieder zerstört
        t_iErr = m_pTransmitter->acceptOrder( pTcpMessage );
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Client::acceptServerMessage( Message * pTcpServerMessage )
  {
    int t_iErr = 0;

    #if defined (__PTHREADS)
      if ( this->status() != EuPTHREAD_WORKING && this->status() != EuPTHREAD_PAUSING )
      {
        t_iErr = EuPTHREAD_ERR_NO_THREAD;
      }
    #else
      if ( this->status() != eWORKING && this->status() != ePAUSING )
      {
        t_iErr = EuTHREAD_ERR_NO_THREAD;
      }
    #endif
    else
    {
      //++++++++++++++++++++++++++++
        this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
        m_MsgQueue.push( pTcpServerMessage );
        this->signalInput();
      //++++++++++++++++++++++++++++
        this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
      #ifdef _DEBUG_THREAD
        std::cout << "tcp::Client::acceptServerMessage(...) - Message uebernommen" << std::endl;
      #endif
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Client::getSocketAdrFromString( SOCKADDR_IN * Socketadress,
                                      const std::string & rsHostnameOrIp ) const
  {
    int             t_iErr = 0;
    unsigned long   t_ulIp = 0;

    // Parameter prüfen
    if ( rsHostnameOrIp.c_str() == NULL_PTR || Socketadress == NULL_PTR) return ERR_PARAMETER_NULL_FORBIDDEN;

    // eine IP in rsHostnameOrIp ?
    t_ulIp = inet_addr( rsHostnameOrIp.c_str() );
    // bei einem Fehler liefert inet_addr den Rückgabewert INADDR_NONE
    if ( t_ulIp != INADDR_NONE )
    {
      Socketadress->sin_addr.s_addr = t_ulIp;
    }
    else
    {
      // Hostnamen in die dazugehörigen IP-Adressen umwandeln
      HOSTENT * t_pHostent = gethostbyname( rsHostnameOrIp.c_str() );
      if ( t_pHostent == NULL_PTR )
      {
        t_iErr = ERR_UNKNOWN_HOSTNAME;
      }
      else
      {
        // die 4 Bytes der IP von t_pHostent nach Socketadress kopieren
        std::memcpy( & ( Socketadress->sin_addr ), t_pHostent->h_addr_list[0], 4 );
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  std::string Client::getHostName() const
  {
    return m_sHostname;
  }
  //---------------------------------------------------------------------------
  int Client::getPeerName( std::string & rsIPAdress, unsigned int & ruiPort ) const
  {
    int   t_iErr = 0;

    rsIPAdress = inet_ntoa( m_Socketadress.sin_addr );
    ruiPort = static_cast<unsigned int> ( ntohs( m_Socketadress.sin_port ) );
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  ActionCode Client::getWaitingServerMsgActionCode() const
  {
    return m_eToWaitingForServer;
  }
  //---------------------------------------------------------------------------
  void Client::processServerMessage( Message * pTcpMessage )
  {
    if ( pTcpMessage )
    {
      if ( pTcpMessage->getActionCode() == ACTION_SYNC_DATA_SIZE )
      {
        unsigned int t_uiDataSize(0);

        std::istringstream( pTcpMessage->getInfo() ) >> t_uiDataSize;
        this->setDataSize( t_uiDataSize );
      }
      delete pTcpMessage;
    }
  }
  //---------------------------------------------------------------------------
  int Client::syncDataSize()
  {
    m_eToWaitingForServer = ACTION_SYNC_DATA_SIZE;
    int  t_iErr = this->send(  NULL_PTR,              // char * pszRawMsg
                              0,                      // unsigned long ulRawMsgSize
                              0,                      // unsigned long uiMessageID
                              ACTION_SYNC_DATA_SIZE,  // int iActionCode
                              "" );                   // const std::string & rstrInfo
    if ( ! t_iErr )
    {
      #ifndef _DEBUG //!!!!!!!!!!!!!! Lies den Kommentar!
        std::time_t    t_StartWait = std::time( NULL ); // Hier bei Debug nicht aktiv, damit es bei Unterbrechung weitergeht
      #endif
      // warten bis Antwort eingetroffen ist
      while ( m_eToWaitingForServer != ACTION_UNDEF )
      {
        Sleep(1);
        #ifndef _DEBUG
          if ( this->TimeOutSec() )
          {
            std::time_t    t_Now = std::time( NULL );
            if ( ( t_Now - t_StartWait ) > this->TimeOutSec() )
            {
              t_iErr = ERR_TIME_OUT;
              m_eToWaitingForServer = ACTION_UNDEF;
            }
          }
        #endif
      }
      //
      /*
      if ( ! t_iErr )
      {
        m_eToWaitingForServer = ACTION_SYNC_DATA_SIZE;
        // 20160309 Warum das???????????????????????????????????????????
        // zwei mal, damit auch gesetzt werden kann( recv(...) blockiert)
        t_iErr = this->send(  0,  // char * pszRawMsg
                              0,  // unsigned long ulRawMsgSize
                              0,  // unsigned long uiMessageID
                              ACTION_SYNC_DATA_SIZE, // int iActionCode
                              "" ); // const std::string & rstrInfo
        if ( ! t_iErr )
        {
          #ifndef _DEBUG
            t_StartWait = std::time(NULL);
          #endif
          // warten bis Antwort eingetroffen ist
          while ( m_eToWaitingForServer != 0 )
          {
            Sleep(1);
            #ifndef _DEBUG
              if ( this->TimeOutSec() )
              {
                t_Now = std::time(NULL);
                if ( ( t_Now - t_StartWait ) > this->TimeOutSec() )
                {
                  t_iErr = ERR_TIME_OUT;
                  m_eToWaitingForServer = ACTION_UNDEF;
                }
              }
            #endif
          }
        }
      }
      //*/
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  Client & Client::operator= ( const Client & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
