//-----------------------------------------------------------------------------
/*!
\file TcpClientReceiver.cpp
\brief Source for ClientReceiver.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpClientReceiver.h"
#include "TcpClient.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
  #define Sleep(x) usleep( (x)* 1000 )
#endif
#ifdef _DEBUG
  #include <iostream>
  #include <sstream>
#endif
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  /*
   * WinSock 2 extension -- manifest constants for shutdown()
   */
  #define SD_RECEIVE      0x00
  #define SD_SEND         0x01
  #define SD_BOTH         0x02
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER DataReceiver
  //---------------------------------------------------------------------------
  ClientReceiver::ClientReceiver()
  : SUPER(),
    m_bHaveConnection(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ClientReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ClientReceiver::ClientReceiver( SOCKET Socket )
  : SUPER( Socket ),
    m_bHaveConnection(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ClientReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ClientReceiver::ClientReceiver (TcpMain * pParent, SOCKET Socket)
  : SUPER( pParent, Socket ),
    m_bHaveConnection(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ClientReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ClientReceiver::ClientReceiver (TcpMain * pParent, SOCKET Socket, unsigned int uiDataSize)
  : SUPER( pParent, Socket, uiDataSize ),
    m_bHaveConnection(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ClientReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ClientReceiver::ClientReceiver( Client * pParent,
                                  SOCKET Socket,
                                  unsigned int uiDataSize )
  : SUPER( pParent, Socket, uiDataSize ),
    m_bHaveConnection(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ClientReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ClientReceiver::~ClientReceiver()
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class DataReceiver' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  ClientReceiver::ClientReceiver( const ClientReceiver & )
  : m_bHaveConnection(true)
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class DataReceiver' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsequence-point"
  #endif
  //---------------------------------------------------------------------------
  void ClientReceiver::work()
  {
    int               t_iErr = 0;
    AbstractMessage * t_pTcpMessage( NULL_PTR );
    bool              t_bWaiting = false;

    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    //^^^^^^^^^^^^^^^^^^^^
    if ( m_bHaveConnection && m_Socket )
    {
      unsigned int    t_uiInstanceDataSize = m_uiDataSize + 1;
      int             t_iSendBytes = 0;

      m_pszSentBuffer = new char[t_uiInstanceDataSize];
      memset( m_pszSentBuffer, 0, m_uiDataSize ); // zuerst alles auf 0 setzten
      // Blockiert!
      t_iSendBytes = recv( m_Socket, m_pszSentBuffer, m_uiDataSize, 0 );
      #if defined (__PTHREADS)
        pthread_testcancel();
      #else
        if ( this->testCancel() )
        {
          delete[] m_pszSentBuffer;
          m_pszSentBuffer = NULL_PTR;
          return;
        }
      #endif
      //^^^^^^^^^^^^^^^^^^^^
      //++++++++++++++++++++++++++++
      this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( t_iSendBytes == 0 || t_iSendBytes == SOCKET_ERROR )
      {
        m_bHaveConnection = false;
        #if defined(_MSC_VER) || defined(__MINGW32__)
          t_iErr = WSAGetLastError();
        #elif defined(__GNUC__)
          t_iErr = errno;
        #endif
        #if defined(_MSC_VER) || defined(__MINGW32__)
        if ( t_iErr == WSAENOTCONN )
        #elif defined(__GNUC__)
        if ( t_iErr == ENOTCONN )
        #endif
        {
          // Socket ist noch nicht verbunden
          t_iErr = 0;
          t_bWaiting = true;
        }
        else
        {
          // Wenn kein Fehler und der Eltern-Client will sich mit dem Server
          // synchronisieren, dann das Warten im Client abbrechen
          Client  * t_pMyParent = dynamic_cast<Client*>( m_pParentTcpMain );

          if ( ! t_iErr && t_pMyParent->getWaitingServerMsgActionCode() == ACTION_SYNC_DATA_SIZE )
          {
            this->initStreamData();
            this->generateNewTcpMessage(  t_pTcpMessage,
                                          NULL_PTR,  // RawMsg
                                          0,         // RawMsgSize
                                          0,         // MsgID
                                          ERR_TO_MUCH_CLIENTS,  // ActionCode
                                          "" );      // Info
            // wird im Parent wieder gelöscht!
            t_iErr = this->acceptMessage( t_pTcpMessage );
          }
          #if defined(_MSC_VER) || defined(__MINGW32__)
            // wenn positiver Wert aus WSAGetLastError() hier auf Minus
            t_iErr = ( t_iErr > 0 ? t_iErr = t_iErr * -1 : t_iErr );
            // Wenn kein Wert aus WSAGetLastError() oder errno
            if ( ! t_iErr ) t_iErr = ERR_WSAECONNABORTED;
          #elif defined(__GNUC__)
            if ( ! t_iErr || t_iErr == ECONNRESET )
            {
              t_iErr = ERR_WSAECONNRESET;
            }
            else if ( t_iErr == EBADF )
            {
              t_iErr = ERR_WSAECONNABORTED;
            }
          #endif
        }
      }
      if ( ! t_bWaiting )
      {
        if ( t_iErr < 0 ||  t_iSendBytes == 0 )
        {
          m_bHaveConnection = false;
        }
        if ( t_iErr != ERR_WSAECONNABORTED )
        {
          if ( t_iErr < 0 )
          {
            this->initStreamData();
            this->generateNewTcpMessage(  t_pTcpMessage,
                                          NULL_PTR, // RawMsg
                                          0,        // RawMsgSize
                                          0,        // MsgID
                                          t_iErr,   // ActionCode
                                          "" );     // Info
            // wird im Parent wieder gelöscht!
            t_iErr = this->acceptMessage( t_pTcpMessage );
          }
          else if ( ! t_iErr && t_iSendBytes > 0 )
          {
            // Wenn alles bisher ok...
            t_iErr = this->processReceivedData( m_pszSentBuffer,  t_iSendBytes);
          }
        }
      }
      delete[] m_pszSentBuffer;
      m_pszSentBuffer = NULL_PTR;
      //++++++++++++++++++++++++++++
      this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
  void ClientReceiver::cleanUp()
  {
    SUPER::cleanUp();
  }
  #endif
  //---------------------------------------------------------------------------
  int ClientReceiver::acceptMessage( AbstractMessage * pTcpMessage )
  {
    int       t_iErr = 0;
    Client  * t_pMyParent = dynamic_cast<Client*>( m_pParentTcpMain );

    if ( pTcpMessage )
    {
      t_iErr = t_pMyParent->acceptServerMessage( dynamic_cast<Message*>( pTcpMessage ) );
    }
    else
    {
      t_iErr = ERR_PARAMETER_NULL_FORBIDDEN;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  void ClientReceiver::generateNewTcpMessage( AbstractMessage *& prTcpMessage,
                                              char * pszTransmitData,
                                              unsigned int uiTransmitDataSize ) const
  {
    prTcpMessage = NULL_PTR;
    #ifdef _DEBUG_TcpMessage
      AbstractMessage  * t_pRetVal = new Message(  pszTransmitData, uiTransmitDataSize );
      t_pRetVal->m_sDebugInstanceInfo = "ClientReceiver::generateNewTcpMessage-1";
      return t_pRetVal;
    #else
      prTcpMessage = new Message(  pszTransmitData, uiTransmitDataSize );
    #endif
  }
  //---------------------------------------------------------------------------
  void ClientReceiver::generateNewTcpMessage( AbstractMessage *& prTcpMessage,
                                              char * pszRawMsg,
                                              unsigned int uiRawMsgSize,
                                              unsigned int uiMessageID,
                                              int iActionCode,
                                              const std::string & rstrInfo ) const
  {
    prTcpMessage = NULL_PTR;
    #ifdef _DEBUG_TcpMessage
      AbstractMessage  * t_pRetVal = new Message( pszRawMsg,
                                                  ulRawMsgSize,
                                                  uiMessageID,
                                                  iActionCode,
                                                  rstrInfo );
      t_pRetVal->m_sDebugInstanceInfo = "ClientReceiver::generateNewTcpMessage-2";
      return t_pRetVal;
    #else
      prTcpMessage = new Message( pszRawMsg,
                                  uiRawMsgSize,
                                  uiMessageID,
                                  iActionCode,
                                  rstrInfo );
    #endif
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
    int ClientReceiver::cancel()
  #else
    void ClientReceiver::cancel()
  #endif
  {
    if ( m_Socket )
    {
      #if defined (__PTHREADS)
        m_eThreadOrder = EuPTHREAD_END;
      #else
        this->toOrder( eExit );
      #endif
      #if defined(_MSC_VER) || defined(__MINGW32__)
        shutdown( m_Socket, SD_BOTH );
        closesocket( m_Socket );
      #elif defined(__GNUC__)
        shutdown( m_Socket, SHUT_RDWR );
        close( m_Socket );
      #endif
      m_Socket = 0;
    }
    #if defined (__PTHREADS)
      return SUPER::cancel();
    #else 
      SUPER::cancel();
    #endif
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Variable wird auf booleschen Wert ('True' oder 'False') gesetzt (Auswirkungen auf Leistungsverhalten möglich)
    #pragma warning( disable : 4800 )
  #endif
  //---------------------------------------------------------------------------
  bool ClientReceiver::connected() const
  {
    return static_cast<bool> ( m_Socket );
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Variable wird auf booleschen Wert ('True' oder 'False') gesetzt (Auswirkungen auf Leistungsverhalten möglich)
    #pragma warning( default : 4800 )
  #endif
  //---------------------------------------------------------------------------
  ClientReceiver & ClientReceiver::operator= ( const ClientReceiver & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
