//-----------------------------------------------------------------------------
/*!
\file TcpMain.cpp
\brief Source for TcpMain
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpMain.h"
#include "EuException.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  unsigned int TcpMain::m_uiTcpMainCounter = 0;
  //---------------------------------------------------------------------------
  #define SUPER EuEndlessPThread
  //---------------------------------------------------------------------------
  TcpMain::TcpMain()
  : m_iSecTimeOut(0),
    m_uiServerPort(0),
    m_uiDataSize( DEFAULT_DATASIZE ),
    m_iAdressFamily(0),
    m_iSocketType(0),
    m_iProtocol(0),
    m_MainSocket(0)
  {
    ++ m_uiTcpMainCounter;
    #ifdef _DEBUG
      m_strDebugInfo = "TcpMain";
    #endif
  }
  //---------------------------------------------------------------------------
  TcpMain::TcpMain( unsigned int uiPort,
                    int iAdressFamily,
                    int iSocketType,
                    int iProtocol,
                    int iSecTimeOut )
  : m_iSecTimeOut( iSecTimeOut ),
    m_uiServerPort( uiPort ),
    m_uiDataSize( DEFAULT_DATASIZE ),
    m_iAdressFamily( iAdressFamily ),
    m_iSocketType( iSocketType ),
    m_iProtocol( iProtocol ),
    m_MainSocket(0)
  {
    int t_iErr = 0;

    if ( ! m_uiServerPort )
    {
      t_iErr = ERR_NO_PORT;
    }
    if ( ! t_iErr ) t_iErr = this->startWinsock();
    if ( t_iErr )
    {
      this->setWSACleanup();
      throw eut::ErrorNException( "TcpMain ctor()", t_iErr, __FILE__, __LINE__ );
    }
    #ifdef _DEBUG
      m_strDebugInfo = "TcpMain";
    #endif
    ++ m_uiTcpMainCounter;
  }
  //---------------------------------------------------------------------------
  TcpMain::TcpMain( unsigned int uiPort,
                    int iAdressFamily,
                    int iSocketType,
                    int iProtocol,
                    unsigned int uiDataSize,
                    int iSecTimeOut )
  : m_iSecTimeOut( iSecTimeOut ),
    m_uiServerPort( uiPort ),
    m_uiDataSize( uiDataSize ),
    m_iAdressFamily( iAdressFamily ),
    m_iSocketType( iSocketType ),
    m_iProtocol( iProtocol ),
    m_MainSocket(0)
  {
    int t_iErr = 0;

    if ( ! m_uiServerPort )
    {
      t_iErr = ERR_NO_PORT;
    }
    if ( ! t_iErr && ! m_uiDataSize )
    {
      t_iErr = ERR_NO_DATASIZE;
    }
    if ( ! t_iErr ) t_iErr = this->startWinsock();
    if ( t_iErr )
    {
      this->setWSACleanup();
      throw eut::ErrorNException( "TcpMain ctor()", t_iErr, __FILE__, __LINE__ );
    }
    #ifdef _DEBUG
      m_strDebugInfo = "TcpMain";
    #endif
    ++ m_uiTcpMainCounter;
  }
  //---------------------------------------------------------------------------
  TcpMain::~TcpMain()
  {
    this->setWSACleanup();
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class EuEndlessPThread' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  TcpMain::TcpMain( const TcpMain & )
  : m_iSecTimeOut(0),
    m_uiServerPort(0),
    m_uiDataSize( DEFAULT_DATASIZE ),
    m_iAdressFamily(0),
    m_iSocketType(0),
    m_iProtocol(0),
    m_MainSocket(0)
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class EuEndlessPThread' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  unsigned int TcpMain::getPortNum() const
  {
    return m_uiServerPort;
  }
  //---------------------------------------------------------------------------
  void TcpMain::setDataSize( unsigned int uiDataSize )
  {
    if ( ! uiDataSize )
    {
      throw eut::ErrorNException( "TcpMain::setDataSize(...)", ERR_NO_DATASIZE, __FILE__, __LINE__ );
    }
    m_uiDataSize = uiDataSize;
  }
  //---------------------------------------------------------------------------
  unsigned int TcpMain::getDataSize() const
  {
    return m_uiDataSize;
  }
  //---------------------------------------------------------------------------
  SOCKET TcpMain::getMainSocket() const
  {
    return m_MainSocket;
  }
  //---------------------------------------------------------------------------
  void TcpMain::setTimeOutSec( int iSecTimeOut )
  {
    m_iSecTimeOut = iSecTimeOut;
  }
  //---------------------------------------------------------------------------
  int TcpMain::TimeOutSec()
  {
    return m_iSecTimeOut;
  }
  //---------------------------------------------------------------------------
  void TcpMain::setWSACleanup() const
  {
    m_uiTcpMainCounter--;
    if ( ! m_uiTcpMainCounter )
    {
      #if defined(_MSC_VER) || defined(__MINGW32__)
        WSACleanup();
      #endif
    }
  }
  //---------------------------------------------------------------------------
  int TcpMain::startWinsock() const
  {
    int   t_iErr = 0;

    #if defined(_MSC_VER) || defined(__MINGW32__)
      WSADATA wsa;
      t_iErr = WSAStartup( MAKEWORD( 2, 0), & wsa );
    #endif
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  TcpMain & TcpMain::operator= ( const TcpMain & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
