//-----------------------------------------------------------------------------
/*!
\file TcpReceiverTransmitter.cpp
\brief Source for ReceiverTransmitter.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpReceiverTransmitter.h"
#include "EuException.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
#ifdef _DEBUG
  #include <iostream>
  #include <sstream>
#endif
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  /*
   * WinSock 2 extension -- manifest constants for shutdown()
   */
  #define SD_RECEIVE      0x00
  #define SD_SEND         0x01
  #define SD_BOTH         0x02
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  ReceiverTransmitter::ReceiverTransmitter()
  : m_Socket(0)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::ReceiverTransmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  ReceiverTransmitter::ReceiverTransmitter( SOCKET Socket )
  : m_Socket( Socket )
  {
    if ( ! Socket )
    {
      throw eut::ErrorNException( "tcp:ReceiverTransmitter ctor()", ERR_PARAMETER_NULL_FORBIDDEN, __FILE__, __LINE__ );
    }
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::ReceiverTransmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  ReceiverTransmitter::~ReceiverTransmitter()
  {
    if ( m_Socket )
    {
      #if defined(_MSC_VER) || defined(__MINGW32__)
        shutdown( m_Socket, SD_BOTH );
        closesocket( m_Socket );
      #elif defined(__GNUC__)
        shutdown( m_Socket, SHUT_RDWR );
        close( m_Socket );
      #endif
      m_Socket = 0;
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class EuEndlessPThread' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  ReceiverTransmitter::ReceiverTransmitter( const ReceiverTransmitter & )
  : m_Socket(0)
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
  // Warnung: base class 'class EuEndlessPThread' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
    int ReceiverTransmitter::takeABreak ()
  #else
    void ReceiverTransmitter::takeABreak ()
  #endif
  {
    throw eut::ErrorNException( "ReceiverTransmitter::takeABreak()", ERR_METHOD_FORBIDDEN, __FILE__, __LINE__ );
  }
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
    int ReceiverTransmitter::finishBreak ()
  #else
    void ReceiverTransmitter::finishBreak ()
  #endif
  {
    throw eut::ErrorNException( "ReceiverTransmitter::finishBreak()", ERR_METHOD_FORBIDDEN, __FILE__, __LINE__ );
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // warning: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wold-style-cast"
    // warning: cast from A to B increases required alignment from 1 to 4 [-Wcast-align]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wcast-align"
  #endif
  //---------------------------------------------------------------------------
  int ReceiverTransmitter::getPeerName( std::string & rstrIPAdress, unsigned int & ruiPort ) const
  {
    int           t_iErr = 0;
    SOCKADDR_IN   t_Peer;
    #if defined(_MSC_VER) || defined(__MINGW32__)
      int         t_iPeer_len;
    #elif defined(__GNUC__)
      socklen_t   t_iPeer_len;
    #endif

    t_iPeer_len = sizeof( t_Peer );
    if ( getpeername( m_Socket, (struct sockaddr*) & t_Peer, &t_iPeer_len) == -1)
    {
      t_iErr = ERR_GETPEERNAME_FAILED;
    }
    else
    {
      rstrIPAdress = inet_ntoa( t_Peer.sin_addr );
      ruiPort = static_cast<unsigned int> ( ntohs( t_Peer.sin_port ) );
      if ( rstrIPAdress == "127.0.0.1" || rstrIPAdress == "LOCALHOST" )
      {
        char    t_HostName[255];
        
        if ( gethostname( t_HostName, sizeof( t_HostName ) ) == 0 )
        {
          #if defined(_MSC_VER) || defined(__MINGW32__)
            PHOSTENT          t_HostInfo;
          #elif defined(__GNUC__)
            struct hostent  * t_HostInfo;
          #endif

          if ( ( t_HostInfo = gethostbyname( t_HostName ) ) != NULL_PTR )
          {
            rstrIPAdress = inet_ntoa( * ( struct in_addr * ) * t_HostInfo->h_addr_list );
          }
        }
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // warning: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic pop
    // warning: cast from A to B increases required alignment from 1 to 4 [-Wcast-align]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  SOCKET ReceiverTransmitter::Socket () const
  {
    return m_Socket;
  }
  //---------------------------------------------------------------------------
  ReceiverTransmitter & ReceiverTransmitter::operator= ( const ReceiverTransmitter & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
