//-----------------------------------------------------------------------------
/*!
\file TcpServer.cpp
\brief Source for Server und Server::ServerReceiverTransmitter
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpServer.h"
#include "TcpTransmitter.h"
#include "TcpServerReceiver.h"
#include "TcpServerAcceptReceiver.h"
#include "EuException.h"
#include <iostream>
#include <sstream>
#if defined(_MSC_VER) || defined(__MINGW32__)
  #include <windows.h>
#elif defined(__GNUC__)
  #include <unistd.h>
  #define Sleep(x) usleep( (x)* 1000 )
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  // >>>>>>>>> Eingebettete Klassen ServerReceiverTransmitter >>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  Server::ServerReceiverTransmitter::ServerReceiverTransmitter()
  : m_pReceiver(NULL_PTR),
    m_pTransmitter(NULL_PTR)
  {
  }
  //---------------------------------------------------------------------------
  Server::ServerReceiverTransmitter::ServerReceiverTransmitter( Transmitter * pTransmitter,
                                                                ServerReceiver * pReceiver)
  : m_pReceiver( pReceiver ),
    m_pTransmitter( pTransmitter )
  {
    if ( ! m_pTransmitter || ! m_pReceiver )
    {
      throw eut::ErrorNException( "ServerReceiverTransmitter ctor()",
                                  ERR_PARAMETER_NULL_FORBIDDEN,
                                  __FILE__,
                                  __LINE__ );
    }
    m_pTransmitter->start();
    m_pReceiver->start();
  }
  //---------------------------------------------------------------------------
  Server::ServerReceiverTransmitter::~ServerReceiverTransmitter()
  {
    if ( m_pTransmitter )
    {
      m_pTransmitter->cancel();
      delete m_pTransmitter;
    }
    if ( m_pReceiver )
    {
      m_pReceiver->cancel();
      delete m_pReceiver;
    }
  }
  //---------------------------------------------------------------------------
  int Server::ServerReceiverTransmitter::send(  char * pszRawMsg,
                                                unsigned int uiRawMsgSize,
                                                unsigned int uiMessageID,
                                                int iActionCode,
                                                const std::string & rstrInfo )
  {
    int       t_iErr = 0;
    Message * t_pMessage = NULL_PTR;

    t_pMessage = new Message( pszRawMsg, uiRawMsgSize, uiMessageID, iActionCode, rstrInfo );
    #ifdef _DEBUG_TcpMessage
    t_pMessage->m_sDebugInstanceInfo = "Server::ServerReceiverTransmitter::send (char * pszRawMsg...)";
    #endif
    t_iErr = this->send( t_pMessage );
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Server::ServerReceiverTransmitter::send( Message * pTcpMessage )
  {
    int   t_iErr = 0;

    if ( ! pTcpMessage )
    {
      t_iErr = ERR_PARAMETER_NULL_FORBIDDEN;
    }
    else
    {
      // t_pMessage wird im m_pTransmitter wieder zerstört
      t_iErr = m_pTransmitter->acceptOrder( pTcpMessage );
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  void Server::ServerReceiverTransmitter::setValid( bool bValid )
  {
    m_pReceiver->setValid( bValid );
  }
  //---------------------------------------------------------------------------
  int Server::ServerReceiverTransmitter::getPeerName( std::string & rstrIPAdress,
                                                      unsigned int & ruiPort ) const
  {
    int   t_iErr(0);

    if ( m_pReceiver )
    {
      t_iErr = m_pReceiver->getPeerName( rstrIPAdress, ruiPort );
    }
    else
    {
      t_iErr = ERR_NO_RECEIVER;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  SOCKET Server::ServerReceiverTransmitter::Socket() const
  {
    SOCKET   t_RetVal = 0;

    if ( m_pReceiver )
    {
      t_RetVal = m_pReceiver->Socket();
    }
    else
    {
      throw eut::ErrorNException( "Server::ServerReceiverTransmitter::Socket()",
                                  ERR_SOCKET_NOT_EXISTING,
                                  __FILE__,
                                  __LINE__ );
    }
    return t_RetVal;
  }
  //---------------------------------------------------------------------------
  // <<<<<<<<< Eingebettete Klassen ServerReceiverTransmitter <<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>>>>> Server >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  #define SUPER TcpMain
  //---------------------------------------------------------------------------
  Server::Server()
    : SUPER(),
    m_ulMaxClients( 0 ),
    m_pAcceptReceiver( NULL_PTR )
  {
    #ifdef _DEBUG
    m_strDebugInfo = "tcp::Server";
    #endif
  }
  //---------------------------------------------------------------------------
  Server::Server( unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  int iSecTimeOut )
    : SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, iSecTimeOut ),
    m_ulMaxClients( 0 ),
    m_pAcceptReceiver( NULL_PTR )
  {
    #ifdef _DEBUG
    m_strDebugInfo = "tcp::Server";
    #endif
  }
  //---------------------------------------------------------------------------
  Server::Server( unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  unsigned int uiDataSize,
                  int iSecTimeOut )
    : SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, uiDataSize, iSecTimeOut ),
    m_ulMaxClients( 0 ),
    m_pAcceptReceiver( NULL_PTR )
  {
    #ifdef _DEBUG
    m_strDebugInfo = "tcp::Server";
    #endif
  }
  //---------------------------------------------------------------------------
  Server::Server( unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  unsigned int uiDataSize,
                  unsigned long ulMaxClients,
                  int iSecTimeOut)
    : SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, uiDataSize, iSecTimeOut ),
    m_ulMaxClients( ulMaxClients ),
    m_pAcceptReceiver( NULL_PTR )
  {
    if ( ! m_ulMaxClients )
    {
      throw eut::ErrorNException( "tcp::Server ctor())", ERR_MAX_CLIENTS_TO_LOW, __FILE__, __LINE__ );
    }
    // eigenen Thread starten
    this->start();
    try
    {
      this->init();
    }
    catch(...)
    {
      this->cancel();
      throw;
    }
    #ifdef _DEBUG
    m_strDebugInfo = "tcp::Server";
    #endif
  }
  //---------------------------------------------------------------------------
  Server::~Server()
  {
    if ( m_pAcceptReceiver )
    {
      m_pAcceptReceiver->cancel();
      delete m_pAcceptReceiver;
      m_pAcceptReceiver = NULL_PTR;
    }
    // alle Objekte in m_MsgQueue löschen
    while ( ! m_MsgQueue.empty() )
    {
      delete m_MsgQueue.front();
      m_MsgQueue.pop();
    }
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class TcpMain' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  Server::Server( const Server & )
  : m_ulMaxClients( 0 ),
    m_pAcceptReceiver( NULL_PTR )
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class TcpMain' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Konvertierung von "xxx" in "yyy", möglicher Datenverlust
    // m_Socketadress.sin_family = m_iAdressFamily;
    #pragma warning( disable : 4244 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsequence-point"
    // warning: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-conversion"
    // warning: implicit conversion loses integer precision: A to B [-Wconversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wconversion"
    // warning: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wold-style-cast"
  #endif
  //---------------------------------------------------------------------------
  void Server::init()
  {
    long          t_lErr = 0;
    SOCKADDR_IN   t_Socketadress;

    // Initialisierung des socketAccept
    m_MainSocket = socket( m_iAdressFamily, m_iSocketType, 0 );
    if ( m_MainSocket == INVALID_SOCKET )
    {
      this->setWSACleanup();
      #if defined(_MSC_VER) || defined(__MINGW32__)
        t_lErr = WSAGetLastError();
      #elif defined(__GNUC__)
        t_lErr = errno;
      #endif
      // wenn positiver Wert aus WSAGetLastError() hier auf Minus
      t_lErr = ( t_lErr > 0 ? t_lErr = t_lErr * -1 : t_lErr );
      throw eut::ErrorNException( "Server init())", static_cast<int> ( t_lErr ), __FILE__, __LINE__ );
    }
    // Initialisieren der Struktur SOCKADDR_IN
    memset( & t_Socketadress, 0, sizeof( SOCKADDR_IN ) ); // zuerst alles auf 0 setzten
    t_Socketadress.sin_family = m_iAdressFamily;
    t_Socketadress.sin_port = htons( m_uiServerPort );
    t_Socketadress.sin_addr.s_addr = INADDR_ANY; // gewisse Compiler brauchen hier ADDR_ANY
    t_lErr = bind( m_MainSocket, (SOCKADDR*) & t_Socketadress, sizeof( SOCKADDR_IN ) );
    if( t_lErr == SOCKET_ERROR )
    {
      #if defined(_MSC_VER) || defined(__MINGW32__)
        t_lErr = WSAGetLastError();
      #elif defined(__GNUC__)
        t_lErr = errno;
      #endif
      this->setWSACleanup();
      // wenn positiver Wert aus WSAGetLastError() hier auf Minus
      t_lErr = ( t_lErr > 0 ? t_lErr = t_lErr * -1 : t_lErr );
      throw eut::ErrorNException( "Server init())", t_lErr, __FILE__, __LINE__ );
    }
    // socketAccept geht in den "listen-mode"
    t_lErr = listen( m_MainSocket, 10 );
    if ( t_lErr == SOCKET_ERROR )
    {
      this->setWSACleanup();
      #if defined(_MSC_VER) || defined(__MINGW32__)
        t_lErr = WSAGetLastError();
      #elif defined(__GNUC__)
        t_lErr = errno;
      #endif
      // wenn positiver Wert aus WSAGetLastError() hier auf Minus
      t_lErr = ( t_lErr > 0 ? t_lErr = t_lErr * -1 : t_lErr );
      throw eut::ErrorNException( "Server init())", t_lErr, __FILE__, __LINE__ );
    }
    // AcceptReceiver initialisieren und den weitermachen lassen
    m_pAcceptReceiver = new ServerAcceptReceiver( this, m_MainSocket );
    m_pAcceptReceiver->start();
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Konvertierung von "xxx" in "yyy", möglicher Datenverlust
    #pragma warning( default : 4244 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic pop
    // implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic pop
    // warning: implicit conversion loses integer precision: A to B [-Wconversion]
    #pragma GCC diagnostic pop
    // warning: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  void Server::work()
  {
    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
    #endif
    this->waitForInput();
    #if defined (__PTHREADS)
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    //^^^^^^^^^^^^^^^^^^^^
    #if defined (__PTHREADS)
      if ( m_eThreadOrder != EuPTHREAD_TAKE_A_BREAK )
    #else
      if ( this->getStatus() == eWORKING )
    #endif
    {
      AbstractMessage * t_pTcpAbstractMessage = NULL_PTR;
      //++++++++++++++++++++++++++++
      this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( ! m_MsgQueue.empty() )
      {
        t_pTcpAbstractMessage = m_MsgQueue.front();
        m_MsgQueue.pop();
      }
      //++++++++++++++++++++++++++++
      this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++

      if ( t_pTcpAbstractMessage )
      {
        Message             * t_pTcpMessage = dynamic_cast<Message*>( t_pTcpAbstractMessage );
        ServerMessage       * t_pTcpServerMessage = dynamic_cast<ServerMessage*>( t_pTcpAbstractMessage );
        ServerAcceptMessage * t_pTcpServerAcceptMessage = dynamic_cast<ServerAcceptMessage*>( t_pTcpAbstractMessage );
        
        if ( t_pTcpServerMessage )
        {
          // wenn Verbindung unterbrochen...
          if (  t_pTcpServerMessage->getActionCode() == ERR_WSAECONNRESET
                ||
                t_pTcpServerMessage->getActionCode() == ERR_WSAECONNABORTED )

          {
            this->removeClient( t_pTcpServerMessage->getSocket() );
          }
          // Msg an ? übergeben
          this->processClientMessage( t_pTcpServerMessage );
        }
        // Wenn neuer Client...
        else if ( t_pTcpServerAcceptMessage )
        {
          //
          /*// Senden ob angenommen oder abgelehnt
          char          * t_pBuffer = new char[ strlen( "READY" ) + 1 ];
          unsigned int    t_uiSentBytes = 0; // Anzahl Bytes der gesendeten t_pMessage
          //*/

          /*
          // wenn maximale Anzahl erreicht...
          if ( m_ClientMap.size() >= m_ulMaxClients )
          {
            // Hier warten wir eine Weile um dem Client Zeit zu geben, seine Meldung zu verarbeiten
            // ToDo: Ist eine schlechte Lösung; mir fällt aber nichts besseres ein!
            Sleep(100);
            t_pTcpServerAcceptMessage->closeSocket(); // socket schliessen, Client merkt das schon
            delete t_pTcpServerAcceptMessage;
            //delete t_pErrorMsg;
          }
          else
          */
          {
            ServerReceiverTransmitter  * t_pNewServerReceiverTransmitter = this->generateNewServerReceiverTransmitter( this,
                                                                                                  t_pTcpServerAcceptMessage->getSocket(),
                                                                                                  m_uiDataSize );
            this->insertNewClient( t_pNewServerReceiverTransmitter );
            if ( m_ClientMap.size() > m_ulMaxClients )
            {
              this->send( t_pTcpServerAcceptMessage->getSocket(),
                          NULL_PTR, // char * pszRawMsg
                          0,        // unsigned int uiRawMsgSize
                          0,        // unsigned long uiMessageID
                          ERR_TO_MUCH_CLIENTS, // int iActionCode
                          "ToMuchClients" );  // const std::string & rstrInfo
              // Hier warten wir eine Weile um dem Client Zeit zu geben, seine Meldung zu verarbeiten
              // ToDo: Ist eine schlechte Lösung; mir fällt aber nichts besseres ein!
              Sleep(1000);
              this->removeClient( t_pTcpServerAcceptMessage->getSocket() );
            }
            delete t_pTcpServerAcceptMessage;
          }
        }
        else if ( t_pTcpMessage )
        {
          this->processClientMessage( t_pTcpMessage );
        }
      }
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
  void Server::cleanUp()
  {
    SUPER::cleanUp();
    // alle Receiver und Transmitter der Clients aus dem Speicher entfernen
    this->clearClientMap();
  }
  //---------------------------------------------------------------------------
  #else
  void Server::cancel()
  {
    SUPER::cancel();
    // alle Receiver und Transmitter der Clients aus dem Speicher entfernen
    this->clearClientMap();
  }
  #endif
  //---------------------------------------------------------------------------
  void Server::insertNewClient( Server::ServerReceiverTransmitter * pNewServerReceiverTransmitter )
  {
    m_ClientMap.insert( std::pair< SOCKET, ServerReceiverTransmitter * >( pNewServerReceiverTransmitter->Socket(), pNewServerReceiverTransmitter ) );
  }
  //---------------------------------------------------------------------------
  Server::ServerReceiverTransmitter * Server::getClient( SOCKET Socket )
  {
    std::map< SOCKET, ServerReceiverTransmitter * >::iterator  t_Iterator = m_ClientMap.find( Socket );

    if ( t_Iterator != m_ClientMap.end() )
    {
      return (*t_Iterator).second;
    }
    return NULL_PTR;
  }
  //---------------------------------------------------------------------------
  int Server::removeClient( SOCKET Socket )
  {
    int                                                         t_iErr = 0;
    std::map< SOCKET, ServerReceiverTransmitter * >::iterator   t_Iterator = m_ClientMap.find( Socket );

    if ( t_Iterator != m_ClientMap.end() )
    {
      //
      /*
      #ifdef _DEBUG
      std::cout << "Vor delete Client_" << std::endl;
      #endif
      //*/
      delete (*t_Iterator).second;
      m_ClientMap.erase( t_Iterator );
    }
    else
    {
      t_iErr = ERR_NO_SOCKETINDEX;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  Server::ServerReceiverTransmitter * Server::generateNewServerReceiverTransmitter( Server * pParent,
                                                                                    SOCKET Socket,
                                                                                    unsigned int iDataSize ) const
  {
    Server          * t_pCastParent = static_cast<Server*>( pParent );
    Transmitter     * t_pTransmitter = new Transmitter( Socket );
    ServerReceiver  * t_pReceiver = new ServerReceiver( t_pCastParent, Socket, iDataSize );
    // Receiver- und TransmitterThread werden im ctor von ServerReceiverTransmitter gestartet
    ServerReceiverTransmitter  * t_pNewServerReceiverTransmitter = new ServerReceiverTransmitter( t_pTransmitter, t_pReceiver );
    return t_pNewServerReceiverTransmitter;
  }
  //---------------------------------------------------------------------------
  void Server::setClientMapReceiverValid( bool bValid )
  {
    for ( std::map< SOCKET, ServerReceiverTransmitter * >::iterator m_ClientMapIterator = m_ClientMap.begin();
          m_ClientMapIterator != m_ClientMap.end();
          ++ m_ClientMapIterator )
    {
      (*m_ClientMapIterator).second->setValid( bValid );
    }
  }
  //---------------------------------------------------------------------------
  void Server::clearClientMap()
  {
    while ( ! m_ClientMap.empty() )
    {
      delete m_ClientMap.begin()->second;
      m_ClientMap.erase( m_ClientMap.begin() );
    }
  }
  //---------------------------------------------------------------------------
  unsigned long Server::getMaxClients() const
  {
    return m_ulMaxClients;
  }
  //---------------------------------------------------------------------------
  void Server::processClientMessage( AbstractMessage * pTcpMessage )
  {
    if ( pTcpMessage )
    {
      // hier wird eine Anfrage nach der Datengröße direkt beantwortet
      ServerMessage  * t_pCastServerMsg = dynamic_cast<ServerMessage*>(  pTcpMessage);
        
      if  ( t_pCastServerMsg
            &&
            t_pCastServerMsg->getActionCode() == ACTION_SYNC_DATA_SIZE
            &&
            this->getClient( t_pCastServerMsg->getSocket() ) )
      {
        std::ostringstream t_ossTemp;

        t_ossTemp << m_uiDataSize;
        this->send( t_pCastServerMsg->getSocket(),
                    NULL_PTR, // char * pszRawMsg
                    0,  // unsigned int uiRawMsgSize
                    0,  // unsigned long uiMessageID
                    ACTION_SYNC_DATA_SIZE, // int iActionCode
                    t_ossTemp.str() );  // const std::string & rstrInfo
      }
      delete pTcpMessage;
    }
  }
  //---------------------------------------------------------------------------
  int Server::send( SOCKET ClientSocket,
                    char * pszRawMsg,
                    unsigned int uiRawMsgSize,
                    unsigned int uiMessageID,
                    int iActionCode,
                    const std::string & rstrInfo )
  {
    ServerMessage  *  t_pServerMsg = new ServerMessage(  ClientSocket,
                                                       pszRawMsg,
                                                       uiRawMsgSize,
                                                       uiMessageID,
                                                       iActionCode,
                                                       rstrInfo );
    return this->send( t_pServerMsg );
  }
  //---------------------------------------------------------------------------
  int Server::send( Message * pTcpMessage )
  {
    ServerMessage  * t_pCastServerMsg = dynamic_cast<ServerMessage*>( pTcpMessage );

    if ( ! t_pCastServerMsg )
    {
      return ERR_NO_SERVER_MSG;
    }

    int                                                         t_iErr = 0;
    std::map< SOCKET, ServerReceiverTransmitter * >::iterator   t_Iterator = m_ClientMap.find( t_pCastServerMsg->getSocket() );

    if ( t_Iterator != m_ClientMap.end() )
    {
      t_iErr = (*t_Iterator).second->send( t_pCastServerMsg );
    }
    else
    {
      t_iErr = ERR_NO_SOCKETINDEX;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( disable : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wunused-variable"
  #endif
  //---------------------------------------------------------------------------
  int Server::acceptClientMessage( AbstractMessage * pTcpMessage )
  {
    int t_iErr = 0;

    #if defined (__PTHREADS)
      if ( this->status() != EuPTHREAD_WORKING && this->status() != EuPTHREAD_PAUSING )
      {
        t_iErr = EuPTHREAD_ERR_NO_THREAD;
      }
    #else
      if ( this->getStatus() != eWORKING && this->status() != ePAUSING )
      {
        t_iErr = EuTHREAD_ERR_NO_THREAD;
      }
    #endif
    else
    {
      //++++++++++++++++++++++++++++
      this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
      #ifdef _DEBUG
        Message              * t_pDEBUG_TcpMessage = dynamic_cast<Message*>( pTcpMessage );
        ServerAcceptMessage  * t_pDEBUG_TcpServerAcceptMessage = dynamic_cast<ServerAcceptMessage*>( pTcpMessage );
        ServerMessage        * t_pDEBUG_TcpServerMessage = dynamic_cast<ServerMessage*>( pTcpMessage );
      #endif
      m_MsgQueue.push( pTcpMessage );
      this->signalInput();
      //++++++++++++++++++++++++++++
      this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
      #ifdef _DEBUG_THREAD
        std::cout << "Server::acceptClientMessage(...) - Message uebernommen" << std::endl;
      #endif
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( default : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  Server & Server::operator= ( const Server & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<<<< Server <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
