//-----------------------------------------------------------------------------
/*!
\file TcpServerReceiver.cpp
\brief Source for ServerReceiver.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpServerReceiver.h"
#include "TcpServer.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
#ifdef _DEBUG
  #include <iostream>
  #include <sstream>
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  /*
   * WinSock 2 extension -- manifest constants for shutdown()
   */
  #define SD_RECEIVE      0x00
  #define SD_SEND         0x01
  #define SD_BOTH         0x02
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER DataReceiver
  //---------------------------------------------------------------------------
  ServerReceiver::ServerReceiver()
  : SUPER(),
    m_bValid(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerReceiver::ServerReceiver( SOCKET Socket )
  : SUPER( Socket ),
    m_bValid(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerReceiver::ServerReceiver( TcpMain * pParent, SOCKET Socket )
  : SUPER( pParent, Socket ),
    m_bValid(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerReceiver::ServerReceiver( TcpMain * pParent, SOCKET Socket, unsigned int uiDataSize )
  : SUPER( pParent, Socket, uiDataSize ),
    m_bValid(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerReceiver::ServerReceiver( Server * pParent, SOCKET Socket, unsigned int uiDataSize )
  : SUPER( pParent, Socket, uiDataSize ),
    m_bValid(true)
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerReceiver::~ServerReceiver()
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class DataReceiver' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  ServerReceiver::ServerReceiver( const ServerReceiver & )
  : m_bValid(true)
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class DataReceiver' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsequence-point"
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-conversion"
  #endif
  //---------------------------------------------------------------------------
  void ServerReceiver::work()
  {
    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    //^^^^^^^^^^^^^^^^^^^^
    if ( m_bValid )
    {
      int             t_iErr = 0,
                      t_iSendBytes = 0;
      unsigned int    t_uiInstanceDataSize = m_uiDataSize + 1;

      m_pszSentBuffer = new char[t_uiInstanceDataSize];
      memset( m_pszSentBuffer, 0, m_uiDataSize ); // zuerst alles auf 0 setzten
      // Blockiert!
      t_iSendBytes = recv( m_Socket, m_pszSentBuffer, m_uiDataSize, 0 );
      #if defined (__PTHREADS)
        pthread_testcancel();
      #else
        if ( this->testCancel() )
        {
          delete[] m_pszSentBuffer;
          m_pszSentBuffer = NULL_PTR;
          return;
        }
      #endif
      //^^^^^^^^^^^^^^^^^^^^
      //++++++++++++++++++++++++++++
      this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( t_iSendBytes <= 0  )
      {
        #if defined(_MSC_VER) || defined(__MINGW32__)
          t_iErr = WSAGetLastError();
          // wenn positiver Wert aus WSAGetLastError() hier auf Minus
          t_iErr = ( t_iErr > 0 ? t_iErr = t_iErr * -1 : t_iErr );
        #elif defined(__GNUC__)
          t_iErr = errno;
          if ( t_iErr == ECONNRESET )
          {
            t_iErr = ERR_WSAECONNRESET;
          }
          else if ( t_iErr == EBADF )
          {
            t_iErr = ERR_WSAECONNABORTED;
          }
        #endif
      }
      if ( m_bValid && m_Socket )
      {
        if ( ! t_iErr && t_iSendBytes > 0 )
        {
          t_iErr = this->processReceivedData( m_pszSentBuffer,  t_iSendBytes);
        }
        else
        {
          AbstractMessage * t_pTcpServerMessage( NULL_PTR );
          
          if ( t_iSendBytes <= 0 && ! t_iErr )
          {
            t_iErr = ERR_WSAECONNRESET;
          }
          m_bValid = false;
          this->initStreamData();
          this->generateNewTcpMessage(  t_pTcpServerMessage,
                                        NULL_PTR, // RawMsg
                                        0,        // RawMsgSize
                                        0,        // MsgID
                                        t_iErr,   // ActionCode
                                        "" );     // Info
          // wird im Parent wieder gelöscht!
          t_iErr = this->acceptMessage( t_pTcpServerMessage );
        }
      }
      delete[] m_pszSentBuffer;
      m_pszSentBuffer = NULL_PTR;
      //++++++++++++++++++++++++++++
      this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic pop
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
  void ServerReceiver::cleanUp()
  {
    SUPER::cleanUp();
  }
  #endif
  //---------------------------------------------------------------------------
  int ServerReceiver::acceptMessage( AbstractMessage  * pTcpServerMessage )
  {
    int      t_iErr = 0;
    Server * t_pMyParent = dynamic_cast<Server*>( m_pParentTcpMain );

    if ( pTcpServerMessage )
    {
      t_iErr = t_pMyParent->acceptClientMessage( dynamic_cast<ServerMessage*>( pTcpServerMessage ) );
    }
    else
    {
      t_iErr = ERR_PARAMETER_NULL_FORBIDDEN;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  void ServerReceiver::generateNewTcpMessage( AbstractMessage *& prTcpServerMessage,
                                              char * pszTransmitData,
                                              unsigned int uiTransmitDataSize ) const
  {
    prTcpServerMessage = NULL_PTR;
    prTcpServerMessage =  new ServerMessage(  m_Socket,
                                              pszTransmitData,
                                              uiTransmitDataSize );
  }
  //---------------------------------------------------------------------------
  void ServerReceiver::generateNewTcpMessage( AbstractMessage *& prTcpServerMessage,
                                              char * pszRawMsg,
                                              unsigned int uiRawMsgSize,
                                              unsigned int uiMessageID,
                                              int iActionCode,
                                              const std::string & rstrInfo ) const
  {
    prTcpServerMessage = NULL_PTR;
    prTcpServerMessage = new ServerMessage( m_Socket,
                                            pszRawMsg,
                                            uiRawMsgSize,
                                            uiMessageID,
                                            iActionCode,
                                            rstrInfo );
  }
  //---------------------------------------------------------------------------
  void ServerReceiver::setValid( bool bValid )
  {
    m_bValid = bValid;
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
    int ServerReceiver::cancel()
  #else
    void ServerReceiver::cancel()
  #endif
  {
    if ( m_Socket )
    {
      #if defined(_MSC_VER) || defined(__MINGW32__)
        shutdown( m_Socket, SD_BOTH );
        closesocket( m_Socket );
      #elif defined(__GNUC__)
        shutdown( m_Socket, SHUT_RDWR );
        close( m_Socket );
      #endif
      m_Socket = 0;
    }
    #ifdef __PTHREADS
      return SUPER::cancel();
    #else
      SUPER::cancel();
    #endif
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  ServerReceiver & ServerReceiver::operator= ( const ServerReceiver & )
  {
      return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
