//-----------------------------------------------------------------------------
/*!
\file TcpTransmitter.cpp
\brief Source for Transmitter.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTransmitter.h"
#include "EuException.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNABORTED 10053L
  #define WSAECONNRESET 10054L
  #define WSAESHUTDOWN 10058L
#endif
#include <sstream>
#ifdef _DEBUG
  #include <iostream>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER ReceiverTransmitter
  //---------------------------------------------------------------------------
  Transmitter::Transmitter ()
  : SUPER()
  {
    #ifdef _DEBUG
      m_strDebugInfo = "Transmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  Transmitter::Transmitter (SOCKET Socket)
  : SUPER( Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "Transmitter";
    #endif
  }
  //---------------------------------------------------------------------------
  Transmitter::~Transmitter()
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class ReceiverTransmitter' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  Transmitter::Transmitter( const Transmitter & )
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class ReceiverTransmitter' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsequence-point"
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-conversion"
  #endif
  //---------------------------------------------------------------------------
  void Transmitter::work()
  {
    Message  * t_pMessage = NULL_PTR;

    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
    #endif
    this->waitForInput();
    #if defined (__PTHREADS)
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    //^^^^^^^^^^^^^^^^^^^^
    #if defined (__PTHREADS)
      if ( m_eThreadOrder != EuPTHREAD_TAKE_A_BREAK )
    #else
      if ( this->getStatus() == eWORKING )
    #endif
    {
      //++++++++++++++++++++++++++++
      this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( ! m_MsgQueue.empty() )
      {
        t_pMessage = m_MsgQueue.front();
        m_MsgQueue.pop();
      }
      //++++++++++++++++++++++++++++
      this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
      if ( t_pMessage )
      {
        int   t_iSendBytes = 0; // Anzahl Bytes der gesendeten t_pMessage
        bool  t_bNOTSOCK = false;

        do
        {
          t_iSendBytes += send( m_Socket,
                                t_pMessage->getTransmitDataPt() + t_iSendBytes,
                                static_cast<int> ( t_pMessage->getTransmitDataSize() ) - t_iSendBytes, 0 );
          if( t_iSendBytes < 0 ) // Wenn send einen Wert < 0 zurück gibt, deutet dies auf einen Fehler hin.
          {
            #if defined(_MSC_VER) || defined(__MINGW32__)
              int t_iErr = WSAGetLastError();
            #elif defined(__GNUC__)
              int t_iErr = errno;
            #endif
            if ( t_iErr == WSAECONNABORTED || t_iErr == WSAESHUTDOWN || t_iErr == WSAENOTSOCK || t_iErr == WSAECONNRESET || t_iErr == EBADF)
            {
              t_bNOTSOCK = true;
            }
            else
            {
              // wenn positiver Wert aus WSAGetLastError() hier auf Minus
              t_iErr = ( t_iErr > 0 ? t_iErr = t_iErr * -1 : t_iErr );
              std::ostringstream      t_ossThrowOut;
              t_ossThrowOut << "Transmitter::work() -> send Error: " << t_iErr;
              throw eut::ErrorNException(t_ossThrowOut.str(), t_iErr, __FILE__, __LINE__ );
            }
          }
        } while( ! t_bNOTSOCK && t_iSendBytes < static_cast<int> ( t_pMessage->getTransmitDataSize() ) );
        delete t_pMessage;
      }
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: operation on 't_iErr' may be undefined [-Wsequence-point]
    #pragma GCC diagnostic pop
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
  void Transmitter::cleanUp ()
  {
    SUPER::cleanUp();
    while ( ! m_MsgQueue.empty() )
    {
      delete m_MsgQueue.front();
      m_MsgQueue.pop();
    }
  }
  //---------------------------------------------------------------------------
  #else
  //---------------------------------------------------------------------------
  void Transmitter::cancel()
  {
    SUPER::cancel();
    while ( ! m_MsgQueue.empty() )
    {
      delete m_MsgQueue.front();
      m_MsgQueue.pop();
    }
  }
  #endif
  //---------------------------------------------------------------------------
  int Transmitter::acceptOrder (Message * pTcpMessage)
  {
    int t_iErr = 0;

    #if defined (__PTHREADS)
      if ( this->status() != EuPTHREAD_WORKING )
      {
        t_iErr = EuPTHREAD_ERR_NO_THREAD;
      }
    #else
      if ( this->getStatus() != eWORKING )
      {
        t_iErr = EuTHREAD_ERR_NO_THREAD;
      }
    #endif
    else
    {
      //++++++++++++++++++++++++++++
        this->setExclusiveAccess();
      //++++++++++++++++++++++++++++
        m_MsgQueue.push( pTcpMessage );
        this->signalInput();
      //++++++++++++++++++++++++++++
        this->unsetExclusiveAccess();
      //++++++++++++++++++++++++++++
      #ifdef _DEBUG_THREAD
        std::cout << "Transmitter::acceptOrder(...) - Auftrag uebernommen" << std::endl;
      #endif
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  Transmitter & Transmitter::operator= ( const Transmitter & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
