#ifndef INC_EUCRITICALSECTION_H
#define INC_EUCRITICALSECTION_H
//-----------------------------------------------------------------------------
/*!
\file EuCriticalSection.h
\brief
\if english
  Header for CriticalSection
\endif

\if german
  Header für CriticalSection
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuToolTypes.h"
#include <mutex>
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
  #ifdef _MSC_VER
    #pragma warning( disable : 4251 )
  #endif
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Is a mutex which can be queried via its lock mode. See isLocked().

    <B>Example</B>
    \code{.cpp}
    void MyClass::makeAnyThing()
    {
      m_MyCriticalSection->lock();
      // An access from a thread to m_pMyPrivateClass->makeManyThings() via this
      // method prevents other threads from accessing this method (or other methods
      // secured with m_MyLocalCriticalSection) before the CriticalSection is
      // finished with unlock() from the first thread.
      this->makeManyThings();
      // catch an exception here if one can be thrown.
      m_MyCriticalSection->unlock();
    }
    \endcode

  \endif

  \if german
    Ist ein Mutex, welcher über sein Lock-Modus abgefragt werden kann. Siehe isLocked().

    <B>Beispiel</B>
    \code{.cpp}
    void MyClass::makeAnyThing()
    {
      m_MyCriticalSection->lock();
      // Ein Zugriff von einem Thread auf m_pMyPrivateClass->makeManyThings() über
      // diese Methode verhindert, dass anderere Threads auf diese Metode (bzw.
      // andere Methoden die mit m_MyLocalCriticalSection abgesichert sind)
      // zugreifen können, bevor die CriticalSection mit unlock() vom ersten Tread
      // beendet ist.
      this->makeManyThings();
      // hier ggf. eine Exception fangen, wenn eine geworfen werden kann.
      m_MyCriticalSection->unlock();
    }
    \endcode
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tool_export_dll CriticalSection
  {
    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        A mutex to protect local variables, for example.
        \sa lock() and unlock()
      \endif

      \if german
        Einen Mutex um z.B. lokale Variablen zu schützen.
        \sa lock() und unlock()
      \endif
      */
      std::mutex m_ExclusiveAccessMutex;
      //-----------------------------------------------------------------------
      /*!
      \if english
        If an CriticalSection blocks access, this attribute is set to true and
        on release it is set to false again.
      \endif

      \if german
        Wenn eine CriticalSection den Zugriff sperrt, wird dieses Attribut
        auf true und beim Freigeben wieder auf false gesetzt.
      \endif
      */
      bool m_bIsLocked;
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor.
      \endif

      \if german
        Konstruktor.
      \endif
      */
      CriticalSection();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~CriticalSection();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copy constructor.
        \param[in] rT The data is copied from this object.
      \endif

      \if german
        Copy-Konstruktor.
        \param[in] rT Von diesem Objekt werden die Daten übernommen.
      \endif
      */
      CriticalSection( const CriticalSection & rT );
      //-----------------------------------------------------------------------

      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Initiates a CriticalSection.
        \attention The CriticalSection must be terminated at some point.
      \endif

      \if german
        Initiiert eine CriticalSection.
        \attention Die CriticalSection muß unbedingt irgendwann wieder beendet
          werden.
      \endif
      */
      void lock();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Terminates a CriticalSection.
        \return A return value < 0 indicates an error.
      \endif

      \if german
        Beendet eine CriticalSection.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      void unlock();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Is the CriticalSection in lock mode?
        \return If true, then in lock mode.
      \endif

      \if german
        Ist die CriticalSection in Sperrmodus?
        \return Wenn true, dann im Sperrmodus.
      \endif
      */
      bool isLocked() const;
      //-----------------------------------------------------------------------

      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator.
        \param[in] rT The data is transferred from this object.
      \endif

      \if german
        Zuweisungsoperator.
        \param[in] rT Von diesem Objekt werden die Daten übernommen.
      \endif
      */
      CriticalSection & operator= ( const CriticalSection & rT );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
  #ifdef _MSC_VER
    #pragma warning( default : 4251 )
  #endif
  //---------------------------------------------------------------------------
} // namespace eut
#endif
