#ifndef INC_EUREPEATINGTHREAD_H
#define INC_EUREPEATINGTHREAD_H
//-----------------------------------------------------------------------------
/*!
\file EuRepeatingThread.h
\brief
\if english
  Header for RepeatingThread
\endif

\if german
  Header für RepeatingThread
\endif
\author Helmut Jakoby
\copyright © 2019 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
// includes
#include "EuToolTypes.h"
#include "EuCriticalSection.h"
#include <thread>
#include <condition_variable>
#include <mutex>
//-----------------------------------------------------------------------------
/* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
#ifdef _MSC_VER
  #pragma warning( disable : 4251 )
#endif
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Base class for a thread that executes its work endlessly until aborted.

    In subclasses, only the virtual method RepeatingThread::work() must be
    overwritten.

    \attention The thread must be terminated after the start with cancel() before
      calling the destructor!
  \endif

  \if german
    Basisklasse für einen Thread der seine Arbeit endlos bis zum Abbruch ausführt.

    In Unterklassen muß lediglich die virtuale Methode RepeatingThread::work()
    überschrieben werden.

    \attention Der Thread muss nach dem start mit cancel() vor dem Aufruf des
      Destruktors beendet werden!
  \endif

  \author Helmut Jakoby
  \copyright © 2010 Helmut Jakoby
  */
  class __tool_export_dll RepeatingThread
  {
    public:
      //============== local enums
      /*!
      \enum EuTHREADERROR
      \brief
      \if english
        Error range = -10301 to -10400
      \endif

      \if german
        Fehlerbereich = -10301 bis -10400
      \endif
      */
      enum EuTHREADERROR
      {

        /*! -10301:
          \if english
            An expected thread is not available or not started.
          \endif

          \if german
            Es ist ein erwarteter Thread nicht vorhanden bzw. nicht gestartet.
          \endif
        */
        EuTHREAD_ERR_NO_THREAD = -10301,
        /*! -10302:
        \if english
          The thread is already started.
        \endif

        \if german
          Der Threads ist schon gestartet.
        \endif
        */
        EuTHREAD_ERR_THREAD_ALREADY_WORKING = -10302,
        /*! -10303:
          \if english
            The thread has been woken from sleep too often.
          \endif

          \if german
            Der Thread wurde zu oft aus dem 'Schlaf' geweckt.
          \endif
        */
        EuTHREAD_ERR_THREAD_BREAK_COUNT_LESS_THAN_ZERO = -10303,
        /*! -10400:
        \if english
          Undefined error.
        \endif

        \if german
          Undefinierter Fehler.
        \endif
        */
        EuTHREAD_ERR_UNKNOWN = -10400,
      };
      //-----------------------------------------------------------------------

      //-----------------------------------------------------------------------
      /*!
      \if english
        To be able to control the thread, it is helpful to know the work status.
      \endif
    
      \if german
        Um den Thread steuern zu können, ist es förderlich, wenn der Arbeitsstatus
        bekannt ist.
      \endif
      */
      enum STATUS
      {
        /*!
        \if english
          The thread has not yet worked.
        \endif

        \if german
          Der Threads hat noch nicht gearbeitet.
        \endif
        */
        eAT_THE_START,
        /*!
        \if english
          The thread is working.
        \endif

        \if german
          Der Threads ist am arbeiten.
        \endif
        */
        eWORKING,
        /*!
        \if english
          The thread is paused.
        \endif

        \if german
          Der Threads pausiert.
        \endif
        */
        ePAUSING,
        /*!
        \if english
          The thread is just finishing its work.
        \endif

        \if german
          Der Threads beendet gerade seiner Arbeit.
        \endif
        */
        eEnding,
        /*!
        \if english
          The thread is finished with its work.
        \endif

        \if german
          Der Threads ist mit seiner Arbeit fertig.
        \endif
        */
        eCOMPLETED
      };
      //-----------------------------------------------------------------------
      /*!
      \if english
        A RepeatingThread can be given instructions after the start.
      \endif

      \if german
        Einem RepeatingThread können nach dem Start Anweisungen übergeben werden.
      \endif
      */
      enum ORDER
      {
        /*!
        \if english
          Undefined.
        \endif

        \if german
          Undefiniert.
        \endif
        */
        eUNDEFINED,
        /*!
        \if english
          Thread should work.
        \endif

        \if german
          Thread soll arbeiten.
        \endif
        */
        eDO_WORK,
        /*!
        \if english
          Thread should end.
        \endif

        \if german
          Thread soll sich beenden.
        \endif
        */
        eExit
      };
      //-----------------------------------------------------------------------

    private:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The working status.
      \endif

      \if german
        Der Arbeitsstatus.
      \endif
      */
      STATUS m_eStatus;
      //-----------------------------------------------------------------------
      /*!
      \if english
        A thread can be given a command which is set here.
      \endif

      \if german
        Einem Thread kann ein Befehl gegeben werden der hier gesetzt wird.
      \endif
      */
      ORDER m_eOrder;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The 'real' thread.
      \endif

      \if german
        Der 'echte' Thread.
      \endif
      */
      std::thread * m_pThread;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Is required to define access protected areas by
        RepeatingThread::insetExclusiveAccess() or
        RepeatingThread::unsetExclusiveAccess().
      \endif

      \if german
        Wird benötigt, um durch RepeatingThread::insetExclusiveAccess() bzw.
        RepeatingThread::unsetExclusiveAccess() zugriffsgeschützte Bereiche
        zu definieren.
      \endif
      */
      CriticalSection m_ExclusiveAccessSection;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Mutex to wait for input.
      \endif

      \if german
        Mutex zum warten auf Eingabe.
      \endif
      \sa waitForInput() and #m_InputCondVar
      */
      std::mutex m_WaitMutex;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Condition_variable to wait for input.
      \endif

      \if german
        Condition_variable zum warten auf Eingabe.
      \endif
      \sa waitForInput(), signalInput() and #m_WaitMutex
      */
      std::condition_variable m_InputCondVar;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Mutex to pause.
      \endif

      \if german
        Mutex um zu pausieren.
      \endif
      \sa threadWork() und #m_BreakCondVar
      */
      std::mutex m_BreakMutex;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Condition_variable to pause
      \endif

      \if german
        Condition_variable um zu pausieren.
      \endif
      \sa threadWork(), finishBreak() and #m_BreakMutex
      */
      std::condition_variable m_BreakCondVar;
      //-----------------------------------------------------------------------
      /*!
      \if english
        If a RepeatingThread is to pause, this counter is incremented. If the
        thread should wake up again, this counter is decremented. At 0 the thread
        is woken up again.

        This enables a nested sleep and wake up.
      \endif

      \if german
        Wenn ein RepeatingThread pausieren soll, wird dieser Zähler incrementiert.
        Wenn der Thread wieder aufwachen soll, wird dieser Zähler decrementiert. Bei
        0 wird der Thread wieder aufgeweckt.

        Damit wird ein geschachteltes schlafen legen und auwecken ermöglicht.
      \endif
      */
      int m_iBreakCounter;
      //-----------------------------------------------------------------------
      /*!
      \if english
        To avoid "lost wakeup" and "spurious wakeups" this variable is necessary.


        - Lost wakeup: The phenomenon of the lost wakeup is that the sender sends
          its notification before the receiver gets to its wait state. The
          consequence is that the notification is lost. The C++ standard describes
          condition variables as a simultaneous synchronisation mechanism: "The
          condition_variable class is a synchronisation primitive that can be used
          to block a thread, or multiple threads at the same time, ...". So the
          notification gets lost, and the receiver is waiting and waiting and ... .
        - Spurious wakeup: It may happen that the receiver wakes up, although no
          notification happened. At a minimum POSIX Threads and the Windows API can
          be victims of these phenomena.

        Found in: https://www.modernescpp.com/index.php/c-core-guidelines-be-aware-of-the-traps-of-condition-variables
      \endif

      \if german
        Um "lost wakeup" und "spurious wakeups" zu vermeiden, ist diese Variable notwendig.

        - Lost Wakeup: Das Phänomen des Lost Wakeup ist, dass der Sender seine
          Benachrichtigung schickt, bevor der Empfänger im Wartezustand ist.
          Als Konsequenz geht dadurch die Benachrichtigung verloren und der
          Empfänger wartet und wartet und ...
        - Spurious Wakeup: Hier passiert es, dass der Empfänger der Nachricht
          aufwacht, obwohl der Sender keine Benachrichtigung geschickt hat.
          Zumindest POSIX Threads und die Windows API können Opfer dieses Phänomens
          sein.
        Gefunde in: https://www.heise.de/developer/artikel/C-Core-Guidelines-Sei-dir-der-Fallen-von-Bedingungsvariablen-bewusst-4063822.html
      \endif
      */
      int m_iDataReady;
      //-----------------------------------------------------------------------

    #ifdef _DEBUG
      public:
        //============== Attribute
        //-----------------------------------------------------------------------
        /*!
        \if english
          Here you can write something in for DEBUG purposes.
        \endif

        \if german
          Hier kann für DEBUG-Zwecke etwas reingeschrieben werden.
        \endif
        */
        std::string m_strDebugInfo;
        //-----------------------------------------------------------------------
    #endif

    public:
      //============== ctors & dtor
      //-----------------------------------------------------------------------
      /*!
      \if english
        Standard constructor.
      \endif

      \if german
        Standartkonstruktor.
      \endif
      */
      RepeatingThread();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
        \attention The method cancel() must be called before the destructor!
          Since cancel() is virtual, it cannot be executed in dtor.
      \endif

      \if german
        Destruktor.
        \attention Es muss vor dem Destruktor die Methode cancel() aufgerufen werden!
          Da cancel() virtual, kann dieses nicht im dtor ausgeführt werden.
      \endif
      */
      virtual ~RepeatingThread();
      //-----------------------------------------------------------------------

    private:
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif
    
      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      RepeatingThread( const RepeatingThread & );
      //-----------------------------------------------------------------------

    protected:
      //============== methods
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method, executed by the thread, calls the method work() in a loop.
      \endif

      \if german
        Diese Methode, vom Thread ausgeführt, ruft in einer Schleife die Methode 
        work() auf.
      \endif
      */
      void threadWork();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Set the status.
        \param[in] eStatus The status to be set.
      \endif

      \if german
        Setzt den Status.
        \param[in] eStatus Der zu setzende Status.
      \endif
      */
      void setStatus( STATUS eStatus );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the current status.
        \return The seted status.
      \endif

      \if german
        Liefert den aktuellen Status.
        \return Der gesetzte Status.
      \endif
      */
      STATUS getStatus();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Accepts the transferred instruction.
        \param[in] eOrder The statement to be set.
      \endif

      \if german
        Übernimmt die übergebene Anweisung.
        \param[in] eOrder Die zu setzende Anweisungen.
      \endif
      */
      void toOrder( ORDER eOrder );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the last transferred instruction.
        \return The current instructions.
      \endif

      \if german
        Liefert die letzte übergebene Anweisung.
        \return Die aktuelle Anweisungen.
      \endif
      */
      ORDER getOrder();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method is executed by the thread and must be overwritten in subclasses.
      \endif

      \if german
        Diese Methode wird vom Thread ausgeführt und muß in Unterklassen überschrieben
        werden.
      \endif
      \sa threadWork()
      */
      virtual void work() = 0;
      //-----------------------------------------------------------------------

    public:
      //============== methods
      //-----------------------------------------------------------------------
      /*!
      \if english
        Starts the thread.
        \return A return value < 0 indicates an error. E.g. if thread is already
          running.
        \attention The thread must be terminated after start() before the destructor
          with cancel().
      \endif

      \if german
        Startet den Thread.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an. Z.B. wenn Thread schon
          läuft.
        \attention Der Thread muss nach start() vor dem Destruktor mittels cancel()
          beendet werden.
      \endif
      */
      virtual int start();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Terminates the thread.
        \attention Must be called before the destructor if started with start()!
          Since cancel() is virtual, it cannot be executed in dtor.
      \endif

      \if german
        Beendet den Thread.
        \attention Muss vor dem Destruktor aufgerufen werden, wenn mit start()
          gestartet! Da cancel() virtual, kann dieses nicht im dtor ausgeführt
          werden.
      \endif
      */
      virtual void cancel();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns true when thread has ended.
        \return see above.
      \endif

      \if german
        Liefert true, wenn Thread beendet wurde.
        \return s.o.
      \endif
      */
      bool testCancel();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the status of the thread.
        \return The status.
      \endif

      \if german
        Liefert den Status des Threads.
        \return Der Status.
      \endif
      */
      RepeatingThread::STATUS status() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Is the thread in an exclusive area?
        \return If true, it's in an exclusive area.
      \endif

      \if german
        Ist der Thread in einem exclusiven Bereich?
        \return Wenn true, dann im exclusiven Bereich.
      \endif
      \sa setExclusiveAccess() and unsetExclusiveAccess()
      */
      bool isInExclusiveAccess() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Initiates a CriticalSection. Must be canceled using unsetExclusiveAccess ().
      \endif

      \if german
        Initiiert eine CriticalSection. Muss mittels unsetExclusiveAccess() wieder
        aufgehoben werden.
      \endif
      \sa #m_ExclusiveAccessSection
      */
      void setExclusiveAccess();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Cancels an initiated CriticalSection if set by setExclusiveAccess()
      \endif

      \if german
        Hebt eine initiierte CriticalSection auf, wenn durch setExclusiveAccess()
        gesetzt.
      \endif
      \sa #m_ExclusiveAccessSection
      */
      void unsetExclusiveAccess();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the pause counter.
      \endif

      \if german
        Liefert den Pausenzähler.
      \endif
      \sa #m_iBreakCounter
      */
      int getBreakCounter();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Initiates a pause of the thread to interrupt the processing.
        Must be ended again with finishBreak().
         \note Since the thread only skips the processing (i.e. is still running)
          a real suspension of the thread can be achieved by using the method
          waitForInput() in the method work() of the derived class.
     \endif

      \if german
        Initiiert ein pausieren des Threads um die Bearbeitung zu unterbrechen.
        Muss durch finishBreak() wieder beendet werden.
        \note Da der Thread die Bearbeitung nur überspringt (also noch läuft)
          kann ein echtes Aussetzen des Threads durch Nutzung der Methode
          waitForInput() in der Methode work() der abgeleiteten Klasse erreicht
          werden.
      \endif
      */
      virtual void takeABreak();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Ends the working pause. Must have been initiated by takeABreak().
      \endif

      \if german
        Beendet die Bearbeitungs-Pause. Muss über takeABreak() initiiert worden sein.
      \endif
      */
      virtual void finishBreak();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Lets the thread wait at a defined position without activity. Must be
        terminated again by signalInput().
        \note This method can be used in the method work() of a derived class to
          suspend the thread. The thread wakes up again with signalInput(), which
          must also be initiated in the derived class.
      \endif

      \if german
        Läßt den Thread an definierter Position ohne Tätigkeit warten. Muss durch
        signalInput() wieder beendet werden.
        \note Diese Methode kann in der Methode work() einer abgeleiteten Klasse
          genutz werden, um den Thread aussetzen zu lasse. Der Thread wacht durch
          signalInput() wieder auf, was auch in der abgeleiteten Klasse initiiert
          werden muss.
      \endif
      */
      void waitForInput();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Ends the wait without activity initiated by waitForInput().
      \endif

      \if german
        Beendet das Warten ohne Tätigkeit, welches durch waitForInput() initiiert
        wurde.
      \endif
      */
      void signalInput();
      //-----------------------------------------------------------------------

    private:
      //============== operators
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator is not available.
      \endif
    
      \if german
        Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      RepeatingThread & operator= ( const RepeatingThread & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
  #ifdef _MSC_VER
    #pragma warning( default : 4251 )
  #endif
  //---------------------------------------------------------------------------
} // namespace eut
#endif
