//-----------------------------------------------------------------------------
/*!
\file EuRepeatingThread.cpp
\brief Source for RepeatingThread
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuRepeatingThread.h"
#include "EuException.h"
#include <iostream>

#if defined(_MSC_VER) || defined(__MINGW32__)
  #include <windows.h>
#elif defined(__GNUC__)
  #include <unistd.h>
  #define Sleep(x) usleep( (x)* 1000 )
#endif
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  RepeatingThread::RepeatingThread()
    : m_eStatus( eAT_THE_START ),
      m_eOrder( eUNDEFINED ),
      m_pThread( NULL_PTR ),
      m_iBreakCounter(0),
      m_iDataReady( 0 )
  {
  }
  //---------------------------------------------------------------------------
  RepeatingThread::~RepeatingThread()
  {
    delete m_pThread;
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::threadWork()
  {
    if ( m_eOrder != eDO_WORK )
    {
      this->setStatus( eAT_THE_START );
      return;
    }
    this->setStatus( eWORKING );
    while ( m_eOrder != eExit )
    {
      std::unique_lock<std::mutex>    t_Lock( m_BreakMutex );

      m_BreakCondVar.wait( t_Lock, [&] { return ! m_iBreakCounter; } );
      if ( m_eOrder != eExit && m_eStatus == eWORKING )
      {
        this->work();
        {
          std::lock_guard<std::mutex> lck( m_WaitMutex );
          m_iDataReady = ( --m_iDataReady < 0 ? 0 : m_iDataReady );
        }
      }
    }
    this->setStatus( eCOMPLETED );
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::setStatus( STATUS eStatus )
  {
    m_eStatus = eStatus;
  }
  //---------------------------------------------------------------------------
  RepeatingThread::STATUS RepeatingThread::getStatus()
  {
    return m_eStatus;
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::toOrder( ORDER eOrder )
  {
    m_eOrder = eOrder;
  }
  //---------------------------------------------------------------------------
  RepeatingThread::ORDER RepeatingThread::getOrder()
  {
    return m_eOrder;
  }
  //---------------------------------------------------------------------------
  int RepeatingThread::start()
  {
    int  t_iErr = 0;

    // nur wenn jungfräulich...
    if ( m_eStatus == eAT_THE_START )
    {
      #ifdef _DEBUG_ENDLESSTHREAD
        std::cout << "-> EuPTHREAD_AT_THE_START" << std::endl;
      #endif
      this->toOrder( eDO_WORK );
      m_pThread = new std::thread( & RepeatingThread::threadWork, this );
      while ( m_eStatus != eWORKING )
      {
        Sleep(1);
      }
    }
    else
    {
      t_iErr = EuTHREAD_ERR_THREAD_ALREADY_WORKING;
    }
    #ifdef _DEBUG_ENDLESSTHREAD
      std::cout << "-> " << m_strDebugInfo.c_str() << " ->" << t_iErr << std::endl;
    #endif
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::cancel()
  {
    this->toOrder( eExit );
    if ( m_iBreakCounter )
    {
      m_iBreakCounter = 0;
      m_BreakCondVar.notify_one();
    }
    {
      std::lock_guard<std::mutex> lck( m_WaitMutex );
      m_iDataReady = 1;
    }
    m_InputCondVar.notify_one();
    while ( m_eStatus != eCOMPLETED )
    {
      Sleep(1);
    }
    m_pThread->join();
    delete m_pThread; m_pThread = NULL_PTR;
  }
  //---------------------------------------------------------------------------
  bool RepeatingThread::testCancel()
  {
    bool    t_bRetVal( m_eOrder == eExit );
    return t_bRetVal;
  }
  //---------------------------------------------------------------------------
  RepeatingThread::STATUS RepeatingThread::status() const
  {
    return m_eStatus;
  }
  //---------------------------------------------------------------------------
  bool RepeatingThread::isInExclusiveAccess() const
  {
    return m_ExclusiveAccessSection.isLocked();
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::setExclusiveAccess()
  {
   m_ExclusiveAccessSection.lock();
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::unsetExclusiveAccess()
  {
    m_ExclusiveAccessSection.unlock();
  }
  //---------------------------------------------------------------------------
  int RepeatingThread::getBreakCounter()
  {
    return m_iBreakCounter;
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::takeABreak()
  {
    m_iBreakCounter++;
    if ( m_iBreakCounter == 1 )
    {
      m_eStatus = ePAUSING;
    }
  }
  //---------------------------------------------------------------------------
  void RepeatingThread::finishBreak()
  {
    // überprüfen, ob m_iBreakCounter nicht schon 0; das wäre ein schwerer Programmierfehler
    if ( m_iBreakCounter )
    {
      m_iBreakCounter--;
      // wenn m_iBreakCounter == 0, dann aufwecken
      if ( ! m_iBreakCounter )
      {
        m_eStatus = eWORKING;
        m_BreakCondVar.notify_one();
      }
    }
    else
    {
      throw eut::ErrorNException( "EuEndlessPThread::finishBreak() to much calls ",
                                  EuTHREAD_ERR_THREAD_BREAK_COUNT_LESS_THAN_ZERO,
                                  __FILE__,
                                  __LINE__ );
    }
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Variable wird auf booleschen Wert ('True' oder 'False') gesetzt (Auswirkungen auf Leistungsverhalten möglich)
    #pragma warning( disable : 4800 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wold-style-cast"
  #endif
  //---------------------------------------------------------------------------
  void RepeatingThread::waitForInput()
  {
    if ( ! m_iDataReady )
    {
      std::unique_lock<std::mutex>    t_Lock( m_WaitMutex );

      m_InputCondVar.wait( t_Lock, [&] { return (bool) m_iDataReady ; } );
    }
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Variable wird auf booleschen Wert ('True' oder 'False') gesetzt (Auswirkungen auf Leistungsverhalten möglich)
    #pragma warning( default : 4800 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: use of old-style cast [-Wold-style-cast]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  void RepeatingThread::signalInput()
  {
    {
      std::lock_guard<std::mutex> lck( m_WaitMutex );
      m_iDataReady++;
    }
    m_InputCondVar.notify_one();
  }
  //---------------------------------------------------------------------------
} // namespace eut
